Imports System.Collections

Class A1
    Class B1(Of Y)
        Class C1(Of X As System.Attribute)
            Class D1

            End Class
            Class D2

            End Class
            Class D3

            End Class
        End Class
        Class C2(Of X1, X2)
            Class D1

            End Class
            Class D2

            End Class
            Class D3

            End Class
        End Class
        Class C3(Of Y)

        End Class
        Enum C4
            value
        End Enum
        Structure C5
            Dim i As String
        End Structure
        Delegate Sub C6()
    End Class
End Class
