Class MethodResolution4
    Shared Function Value() As Integer
        Return 0
    End Function
    Shared Function Main() As Integer
        Dim v As Integer = Value
        Return v
    End Function
End Class