'Test emitting local variables.
Class EmitLocalVariables1
    Private Shared Sub Main()
        Dim bo As Boolean
        val_boolean(bo)
        ref_boolean(bo)
        Dim by As Byte
        val_byte(by)
        ref_byte(by)
        Dim ch As Char
        val_char(ch)
        ref_char(ch)
        Dim dt As Date
        val_date(dt)
        ref_date(dt)
        Dim de As Decimal
        val_decimal(de)
        ref_decimal(de)
        Dim db As Double
        val_double(db)
        ref_double(db)
        Dim i32 As Integer
        val_integer(i32)
        ref_integer(i32)
        Dim i16 As Short
        val_short(i16)
        ref_short(i16)
        Dim i64 As Long
        val_long(i64)
        ref_long(i64)
        Dim str As String
        val_string(str)
        ref_string(str)
        Dim si As Single
        val_single(si)
        ref_single(si)
        Dim ui16 As UShort
        val_ushort(ui16)
        ref_ushort(ui16)
        Dim ui32 As UInteger
        val_uinteger(ui32)
        ref_uinteger(ui32)
        Dim ui64 As ULong
        val_ulong(ui64)
        ref_ulong(ui64)
    End Sub
    Shared Sub ref_boolean(ByRef b As Boolean)

    End Sub
    Shared Sub val_boolean(ByVal b As Boolean)

    End Sub

    Shared Sub ref_byte(ByRef b As Byte)

    End Sub
    Shared Sub val_byte(ByVal b As Byte)

    End Sub

    Shared Sub ref_char(ByRef b As Char)

    End Sub
    Shared Sub val_char(ByVal b As Char)

    End Sub

    Shared Sub ref_date(ByRef b As Date)

    End Sub
    Shared Sub val_date(ByVal b As Date)

    End Sub

    Shared Sub ref_decimal(ByRef b As Decimal)

    End Sub
    Shared Sub val_decimal(ByVal b As Decimal)

    End Sub

    Shared Sub ref_double(ByRef b As Double)

    End Sub
    Shared Sub val_double(ByVal b As Double)

    End Sub

    Shared Sub ref_short(ByRef b As Short)

    End Sub
    Shared Sub val_short(ByVal b As Short)

    End Sub

    Shared Sub ref_integer(ByRef b As Integer)

    End Sub
    Shared Sub val_integer(ByVal b As Integer)

    End Sub

    Shared Sub ref_long(ByRef b As Long)

    End Sub
    Shared Sub val_long(ByVal b As Long)

    End Sub

    Shared Sub ref_sbyte(ByRef b As SByte)

    End Sub
    Shared Sub val_sbyte(ByVal b As SByte)

    End Sub

    Shared Sub ref_single(ByRef b As Single)

    End Sub
    Shared Sub val_single(ByVal b As Single)

    End Sub

    Shared Sub val_string(ByVal b As String)

    End Sub
    Shared Sub ref_string(ByRef b As String)

    End Sub
    Shared Sub ref_ushort(ByRef b As UShort)

    End Sub
    Shared Sub val_ushort(ByVal b As UShort)

    End Sub

    Shared Sub ref_uinteger(ByRef b As UInteger)

    End Sub
    Shared Sub val_uinteger(ByVal b As UInteger)

    End Sub

    Shared Sub ref_ulong(ByRef b As ULong)

    End Sub
    Shared Sub val_ulong(ByVal b As ULong)

    End Sub

End Class