Option Explicit On
Option Strict On

Class X
    Public Shared Sub Main()
        Dim b As Byte
        Dim sb As SByte
        Dim s As Short
        Dim us As UShort
        Dim i As Integer
        Dim ui As UInteger
        Dim l As Long
        Dim ul As ULong
        Dim dt As Date
        Dim chr As Char
        Dim str As String
        Dim bool As Boolean
        Dim sng As Single
        Dim dbl As Double
        Dim dec As Decimal

        Dim y As Byte = s
    End Sub
End Class