Imports System
Imports System.Collections
Imports System.Reflection

Namespace PartialAccess1
    Class Test
        Shared Function Main() As Integer

        End Function
    End Class
    Partial Class T

    End Class
    Partial Public Class T

    End Class
End Namespace