Imports System
Imports System.Collections
Imports System.Reflection

Namespace MethodAttributes2
    MustInherit Class Test
        MustOverride Overrides Function ToString() As String
        Shared Function Main() As Integer

        End Function
    End Class
End Namespace