MustInherit Class ArgumentTypeInference
    Property c(ByVal vvv As Integer) As Boolean
        Get

        End Get
        Set(ByVal c As Boolean)

        End Set
    End Property
    WriteOnly Property d(ByVal vvv As Integer) As Boolean
        Set(ByVal d As Boolean)

        End Set
    End Property
End Class