Class TypeArguments6
    Class Test(Of V)
        Sub Test()

        End Sub
        Function Test2() As v

        End Function
        Sub Test3(ByVal param1 As v)

        End Sub
    End Class
End Class