Imports System
Imports System.Collections
Imports System.Reflection

Class GenericMethod8
    Sub A(ByVal P1 As Generic.List(Of String))
        Dim l As generic.list(Of String)
        l.addrange(p1)
    End Sub

    Shared Sub Main()
    End Sub
End Class