Class Do1
	Sub Test
		Dim i As Integer

		Do Until True
			i = 1
		Loop			

		Do While True
			I = 2
		Loop

		do
			i = 3
		loop until true

		do 
			i = 4
		loop while true

		do
			i = 5
		loop
	End Sub
End Class
