public Class DecimalConstant1
	public Const d1 As Decimal = -1
	public Const d2 As Decimal = system.Decimal.maxvalue
	public const d3 As Decimal = system.Decimal.minvalue
	public Const d4 As Decimal = 1
End Class