Partial Class Partial1
    Property T() As Integer
        Get

        End Get
        Set(ByVal value As Integer)

        End Set
    End Property
End Class
Partial Class Partial1
    Property P() As Integer
        Get

        End Get
        Set(ByVal value As Integer)

        End Set
    End Property
End Class