'Test nested #if
#If False Then
#if true then
class ConditionalIf2__1
end class
#else
class ConditionalIf2__2
end class
#end if
class ConditionalIf2_1
end class
#ElseIf False Then
#if true then
class ConditionalIf2__3
end class
#else
class ConditionalIf2__4
end class
#end if
class ConditionalIf2_3
end class
#Else
#If True Then
class ConditionalIf2__5
end class
#Else
Class ConditionalIf2__6
End Class
#End If
Class ConditionalIf2_2
End Class
#End If