Class ParamArray1
    Sub Test1(ByVal ParamArray Values() As String)

    End Sub
    Sub Test2(ByVal OtherParam As String, ByVal ParamArray Values() As Date)

    End Sub
    Sub Test3(ByVal a As Object, ByVal ParamArray b() As Object)
        Test3("")
    End Sub
End Class

