'Test events
Namespace Events1
    Class a_event
        Event a()
    End Class
    Class b_event
        Event b(ByVal bb As String)
    End Class
    Class c_event
        Event c(ByRef cc As String)
        Sub test()
        End Sub
    End Class
End Namespace