Option Compare Text
Class OptionCompareText2
    Public Event NoCrashHere()
    Delegate Sub Foo()
    Class Nested

    End Class
    Enum E
        i
    End Enum
    Structure S
        Dim i As Integer
    End Structure
End Class
Module OptionCompareText2_Module
End Module