Class MethodResolution5
    Shared Function Main() As Integer
        Dim tmp As String
        tmp = String.Format("{0}{1}{2}", "first", "second", "third")
        Return 0
    End Function
End Class