Public Class MemberAttributes1
    Sub A(ByVal i As Integer)

    End Sub
    Sub A(ByVal i As Long)

    End Sub

    Public Sub Remove(ByVal i As Integer)

    End Sub

    Public Sub Remove(ByVal key As String)

    End Sub
End Class