Module Mod3
    Public Function M(ByVal i As Integer) As Integer

    End Function
    Public Function M(ByVal l As Long) As Integer

    End Function

End Module
Class Module3
    Shared Function Main() As Integer
        Return M(1)
    End Function
End Class