Imports System
Imports System.Collections
Imports System.Reflection

Namespace DefaultFunctionVariable1
    Class Test
        Shared Function Main() As Integer
            Main = 0
        End Function
    End Class
End Namespace