'Test local variable assignment
Class LocalVariableAssignment1
    Dim b As Double
    Sub Main()
        Dim bo As Boolean
        Dim by As Byte
        Dim ch As Char
        Dim dt As Date
        Dim de As Decimal
        Dim db As Double
        Dim i32 As Integer
        Dim i16 As Short
        Dim i64 As Long
        Dim str As String
        Dim si As Single
        Dim ui16 As UShort
        Dim ui32 As UInteger
        Dim ui64 As ULong
        'b = bo
        'bo = b
        ''bo = true
        b = system.math.pi
        test2()
    End Sub
    Sub Test2()

    End Sub
End Class