Class ia64_1

	Shared Sub Main
		EmitLoadI4Value (2)
	End Sub

	Shared Sub EmitLoadI4Value(ByVal I As Integer)
		If I >= SByte.MinValue AndAlso I <= SByte.MaxValue Then
		End If
    	End Sub
End Class
