Imports System
Imports System.Collections
Imports System.Reflection

Namespace For4
    Class Test
        Shared Function Main() As Integer
            Dim i, l As Integer
            For i = l To 2

            Next
        End Function
    End Class
End Namespace