Namespace MethodInvocation10
    Class Test
        Shared Function Main() As Integer
            Dim i As Integer
            Dim s As String
            s = i.tostring
            s = 1.tostring
            Return 0
        End Function
    End Class
End Namespace