Imports VIA.Workcamps
Imports VIA.Core

Public Class kamp
    Private Sub Page_Load(ByVal sender As System.Object, ByVal e As System.EventArgs)
	Dim camps As Object
        If IsNothing(camps) Then
	        Exit Select
        End If
    End Sub
End Class
