 '------------------------------------------------------------------------------
 ' <autogenerated>
 '     This code was generated by a tool.
 '     Mono Runtime Version: 2.0.50727.42
 '
 '     Changes to this file may cause incorrect behavior and will be lost if 
 '     the code is regenerated.
 ' </autogenerated>
 '------------------------------------------------------------------------------
 
 Option Explicit On
 Option Strict Off
 
 

Imports System
Imports System.Collections
Imports System.Collections.Specialized
Imports System.Configuration
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Web
Imports System.Web.Caching
Imports System.Web.SessionState
Imports System.Web.Security
Imports System.Web.Profile
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.HtmlControls


Partial Class _Default
    Inherits System.Web.UI.Page
End Class

Public Class _Default
    Implements System.Web.SessionState.IRequiresSessionState

    Protected Sub Button1_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Button1.Click
        Label1.Text = "Hello World"
    End Sub

    Protected _bctrl_4 As System.Web.UI.HtmlControls.HtmlHead

    Protected _bctrl_5 As System.Web.UI.HtmlControls.HtmlTitle

    Protected form1 As System.Web.UI.HtmlControls.HtmlForm

    Protected WithEvents Button1 As System.Web.UI.WebControls.Button

    Protected Label1 As System.Web.UI.WebControls.Label

    Protected ReadOnly Property ApplicationInstance() As System.Web.HttpApplication
        Get
            Return CType(Me.Context.ApplicationInstance, System.Web.HttpApplication)
        End Get
    End Property

    Protected ReadOnly Property Profile() As System.Web.Profile.DefaultProfile
        Get
            Return CType(Me.Context.Profile, System.Web.Profile.DefaultProfile)
        End Get
    End Property
End Class
 Namespace ASP
     
     Public Class default_aspx
         Inherits _Default
         
         Private Shared __initialized As Boolean = false
         
         Public Sub New()
             MyBase.New
             CType(Me, System.Web.UI.Page).AppRelativeVirtualPath = "~/Default.aspx"
             If (default_aspx.__initialized = false) Then
                 default_aspx.__initialized = true
             End If
         End Sub
         
         Protected Overrides ReadOnly Property SupportAutoEvents() As Boolean
             Get
                 Return false
             End Get
         End Property
         
         Public Overrides ReadOnly Property TemplateSourceDirectory() As String
             Get
                 If (Not (Me.Parent) Is Nothing) Then
                     Return Me.Parent.TemplateSourceDirectory
                 End If
                 Return "/"
             End Get
         End Property
         
         Private Sub __BuildControlTree(ByVal __ctrl As default_aspx)
             Me.InitializeCulture()
             Dim __parser As System.Web.UI.IParserAccessor = CType(__ctrl, System.Web.UI.IParserAccessor)
            __parser.AddParsedSubObject(New System.Web.UI.LiteralControl("<!DOCTYPE html PUBLIC ""-//W3C//DTD XHTML 1.0 Transitional//EN""""http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"">" & Microsoft.VisualBasic.ChrW(10) & Microsoft.VisualBasic.ChrW(10) & "<html xmlns=""http://www.w3.org/1999/xhtml"">" & Microsoft.VisualBasic.ChrW(10)))
             Me.__BuildControl__bctrl_4()
             __parser.AddParsedSubObject(Me._bctrl_4)
             __parser.AddParsedSubObject(New System.Web.UI.LiteralControl(""&Microsoft.VisualBasic.ChrW(10)&"<body>"&Microsoft.VisualBasic.ChrW(10)&"    "))
             Me.__BuildControl_form1()
             __parser.AddParsedSubObject(Me.form1)
            __parser.AddParsedSubObject(New System.Web.UI.LiteralControl("" & Microsoft.VisualBasic.ChrW(10) & "</body>" & Microsoft.VisualBasic.ChrW(10) & "</html>" & Microsoft.VisualBasic.ChrW(10)))
         End Sub
         
         Private Function __BuildControl__bctrl_4() As System.Web.UI.Control
             Dim __ctrl As System.Web.UI.HtmlControls.HtmlHead
             __ctrl = New System.Web.UI.HtmlControls.HtmlHead("head")
             Me._bctrl_4 = __ctrl
             Me.__BuildControl__bctrl_5()
             Dim __parser As System.Web.UI.IParserAccessor = CType(__ctrl, System.Web.UI.IParserAccessor)
             __parser.AddParsedSubObject(Me._bctrl_5)
             Return __ctrl
         End Function
         
         Private Function __BuildControl__bctrl_5() As System.Web.UI.Control
             Dim __ctrl As System.Web.UI.HtmlControls.HtmlTitle
             __ctrl = New System.Web.UI.HtmlControls.HtmlTitle()
             Me._bctrl_5 = __ctrl
             Dim __parser As System.Web.UI.IParserAccessor = CType(__ctrl, System.Web.UI.IParserAccessor)
             __parser.AddParsedSubObject(New System.Web.UI.LiteralControl("Untitled Page"))
             Return __ctrl
         End Function
         
         Private Function __BuildControl_form1() As System.Web.UI.Control
             Dim __ctrl As System.Web.UI.HtmlControls.HtmlForm
             __ctrl = New System.Web.UI.HtmlControls.HtmlForm()
             Me.form1 = __ctrl
             __ctrl.ID = "form1"
             Dim __parser As System.Web.UI.IParserAccessor = CType(__ctrl, System.Web.UI.IParserAccessor)
             __parser.AddParsedSubObject(New System.Web.UI.LiteralControl(""&Microsoft.VisualBasic.ChrW(10)&"    <div>"&Microsoft.VisualBasic.ChrW(10)&"        "))
             Me.__BuildControl_Button1()
             __parser.AddParsedSubObject(Me.Button1)
             __parser.AddParsedSubObject(New System.Web.UI.LiteralControl(""&Microsoft.VisualBasic.ChrW(10)&"        "))
             Me.__BuildControl_Label1()
             __parser.AddParsedSubObject(Me.Label1)
             __parser.AddParsedSubObject(New System.Web.UI.LiteralControl("</div>"&Microsoft.VisualBasic.ChrW(10)&"    "))
             Return __ctrl
         End Function
         
         Private Function __BuildControl_Button1() As System.Web.UI.Control
             Dim __ctrl As System.Web.UI.WebControls.Button
             __ctrl = New System.Web.UI.WebControls.Button()
             Me.Button1 = __ctrl
             __ctrl.ApplyStyleSheetSkin(Me)
             __ctrl.Text = "Button"
             __ctrl.ID = "Button1"
             Return __ctrl
         End Function
         
         Private Function __BuildControl_Label1() As System.Web.UI.Control
             Dim __ctrl As System.Web.UI.WebControls.Label
             __ctrl = New System.Web.UI.WebControls.Label()
             Me.Label1 = __ctrl
             __ctrl.ApplyStyleSheetSkin(Me)
             __ctrl.Text = "Label"
             __ctrl.ID = "Label1"
             Return __ctrl
         End Function
         
         Protected Overrides Sub FrameworkInitialize()
             MyBase.FrameworkInitialize()
             Me.TraceModeValue = System.Web.TraceMode.SortByTime
             Me.Request.ValidateInput()
             Me.__BuildControlTree(Me)
         End Sub
         
         Public Overrides Function GetTypeHashCode() As Integer
             Return 953835190
         End Function
     End Class
 End Namespace
