Imports System
Imports System.Collections
Imports System.Reflection

Namespace FieldAccess1
    Public Class Test
        ReadOnly Field As Integer
        Shared Sub Main()

        End Sub
    End Class
End Namespace