'
' Globals.vb
'
' Authors:
'   Rolf Bjarne Kvinge (RKvinge@novell.com>
'
' Copyright (C) 2007 Novell (http://www.novell.com)
'
' Permission is hereby granted, free of charge, to any person obtaining
' a copy of this software and associated documentation files (the
' "Software"), to deal in the Software without restriction, including
' without limitation the rights to use, copy, modify, merge, publish,
' distribute, sublicense, and/or sell copies of the Software, and to
' permit persons to whom the Software is furnished to do so, subject to
' the following conditions:
' 
' The above copyright notice and this permission notice shall be
' included in all copies or substantial portions of the Software.
' 
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
' EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
' MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
' NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
' LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
' OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
' WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
'
Namespace Microsoft.VisualBasic
    <Microsoft.VisualBasic.CompilerServices.StandardModule()> _
    Public NotInheritable Class Globals
        Public Shared ReadOnly Property ScriptEngine() As String
            Get
                Return "VB"
            End Get
        End Property

        Public Shared ReadOnly Property ScriptEngineBuildVersion() As Integer
            Get
#If NET_VER >= 4.0 Then
                Return 30319
#Else
                Return 50727
#End If
            End Get
        End Property

        Public Shared ReadOnly Property ScriptEngineMajorVersion() As Integer
            Get
#If NET_VER >= 4.0 Then
                Return 10
#Else
                Return 8
#End If
            End Get
        End Property

        Public Shared ReadOnly Property ScriptEngineMinorVersion() As Integer
            Get
                Return 0
            End Get
        End Property
    End Class
End Namespace

