Imports System
Imports System.Collections
Imports System.Reflection

Namespace Attribute2
    <Flags()> _
    <System.Serializable()> _
    Enum Test
        Value
    End Enum
    Class Test2
        Shared Function Main() As Integer

        End Function
    End Class
End Namespace