Option Explicit On
Option Strict On
Option Compare Binary
REM Option Infer Off

Public Class Test1
    Public Shared Sub Main()
        a = 3
        Dim a As Integer
        System.Console.WriteLine(a)
    End Sub
End Class