MustInherit Class ArgumentTypeInference
    Sub a(ByVal a As Integer)

    End Sub
    Property c(ByVal c As Integer) As Boolean
        Get
            Return False
        End Get
        Set(ByVal value As Boolean)

        End Set
    End Property
End Class