Module IdentifierFail1
    ' not a valid identifier as per spec
    ' but accepted by .net 1.0 
    Function __() As Object
    End Function

    Function Main() As Integer
    End Function
End Module
