'Author:
'   Maverson Eduardo Schulze Rosa (maverson@gmail.com)
'
' GrupoTIC - UFPR - Federal University of Paraná

Option Strict On
Imports System

Module Test
    Interface A
        Property Prop$()
    End Interface

    Public Function Main() As Integer
    End Function
End Module
