'------------------------------------------------------------------------------
' <autogenerated>
'     This code was generated by a tool.
'     Mono Runtime Version: 2.0.50727.42
'
'     Changes to this file may cause incorrect behavior and will be lost if 
'     the code is regenerated.
' </autogenerated>
'------------------------------------------------------------------------------

Option Explicit On
Option Strict Off

Imports System
Imports System.Collections
Imports System.Collections.Specialized
Imports System.Configuration
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Web
Imports System.Web.Caching
Imports System.Web.SessionState
Imports System.Web.Security
Imports System.Web.Profile
Imports System.Web.UI
Imports System.Web.UI.WebControls
'Imports System.Web.UI.HtmlControls

Namespace VIA.Web
    Public Class _default8
        Inherits System.Web.UI.Page
    End Class
End Namespace

Namespace ASP

    Public Class default_aspx
        Inherits VIA.Web._default8
        Implements System.Web.SessionState.IRequiresSessionState

        Private Shared __initialized As Boolean = False

        Protected Form1 As System.Web.UI.HtmlControls.HtmlForm

        Public Sub New()
            MyBase.New()
            CType(Me, VIA.Web._default8).AppRelativeVirtualPath = "~/default.aspx"
            If (ASP.default_aspx.__initialized = False) Then
                ASP.default_aspx.__initialized = True
            End If
        End Sub

        Protected Overrides ReadOnly Property SupportAutoEvents() As Boolean
            Get
                Return False
            End Get
        End Property

        Protected ReadOnly Property ApplicationInstance() As System.Web.HttpApplication
            Get
                Return CType(Me.Context.ApplicationInstance, System.Web.HttpApplication)
            End Get
        End Property

        Public Overrides ReadOnly Property TemplateSourceDirectory() As String
            Get
                If (Me.Parent Is Nothing) = False Then
                    Return Me.Parent.TemplateSourceDirectory
                End If
                Return "/"
            End Get
        End Property

        Protected ReadOnly Property Profile() As System.Web.Profile.DefaultProfile
            Get
                Return CType(Me.Context.Profile, System.Web.Profile.DefaultProfile)
            End Get
        End Property

        Private Sub __BuildControlTree(ByVal __ctrl As default_aspx)
            Me.InitializeCulture()
            Dim __parser As System.Web.UI.IParserAccessor = CType(__ctrl, System.Web.UI.IParserAccessor)
            __parser.AddParsedSubObject(New System.Web.UI.LiteralControl("<!DOCTYPE HTML PUBLIC ""-//W3C//DTD HTML 4.0 Transitional//EN"">" & Microsoft.VisualBasic.ChrW(10) & "<HTML>" & Microsoft.VisualBasic.ChrW(10) & Microsoft.VisualBasic.ChrW(9) & "<HEAD>" & Microsoft.VisualBasic.ChrW(10) & Microsoft.VisualBasic.ChrW(9) & Microsoft.VisualBasic.ChrW(9) & "<title>default</title>" & Microsoft.VisualBasic.ChrW(10) & Microsoft.VisualBasic.ChrW(9) & Microsoft.VisualBasic.ChrW(9) & "<meta name=""GENERATOR"" content=""Microsoft Visual Studio .NET 7.1"">" & Microsoft.VisualBasic.ChrW(10) & Microsoft.VisualBasic.ChrW(9) & Microsoft.VisualBasic.ChrW(9) & "<meta name=""CODE_LANGUAGE"" content=""Visual Basic .NET 7.1"">" & Microsoft.VisualBasic.ChrW(10) & Microsoft.VisualBasic.ChrW(9) & Microsoft.VisualBasic.ChrW(9) & "<meta name=""vs_defaultClientScript"" content=""JavaScript"">" & Microsoft.VisualBasic.ChrW(10) & Microsoft.VisualBasic.ChrW(9) & Microsoft.VisualBasic.ChrW(9) & "<meta name=""vs_targetSchema"" content=""http://schemas.microsoft.com/intellisense/ie3-2nav3-0"">" & Microsoft.VisualBasic.ChrW(10) & Microsoft.VisualBasic.ChrW(9) & "</HEAD>" & Microsoft.VisualBasic.ChrW(10) & Microsoft.VisualBasic.ChrW(9) & "<body MS_POSITIONING=""FlowLayout"">" & Microsoft.VisualBasic.ChrW(10) & Microsoft.VisualBasic.ChrW(9) & Microsoft.VisualBasic.ChrW(9)))
            Me.__BuildControl_Form1()
            __parser.AddParsedSubObject(Me.Form1)
            __parser.AddParsedSubObject(New System.Web.UI.LiteralControl("" & Microsoft.VisualBasic.ChrW(10) & Microsoft.VisualBasic.ChrW(9) & "</body>" & Microsoft.VisualBasic.ChrW(10) & "</HTML>" & Microsoft.VisualBasic.ChrW(10)))
            __ctrl.SetRenderMethodDelegate(AddressOf Me.__RenderTree)
        End Sub

        Private Function __BuildControl_Form1() As System.Web.UI.Control
            Dim __ctrl As System.Web.UI.HtmlControls.HtmlForm
            __ctrl = New System.Web.UI.HtmlControls.HtmlForm()
            Me.Form1 = __ctrl
            __ctrl.Method = "post"
            __ctrl.ID = "Form1"
            Dim __parser As System.Web.UI.IParserAccessor = CType(__ctrl, System.Web.UI.IParserAccessor)
            __parser.AddParsedSubObject(New System.Web.UI.LiteralControl("" & Microsoft.VisualBasic.ChrW(10) & Microsoft.VisualBasic.ChrW(9) & Microsoft.VisualBasic.ChrW(9)))
            Return __ctrl
        End Function

        Protected Overrides Sub FrameworkInitialize()
            MyBase.FrameworkInitialize()
            Me.TraceModeValue = System.Web.TraceMode.SortByTime
            Me.Request.ValidateInput()
            Me.MaintainScrollPositionOnPostBack = True
            Me.__BuildControlTree(Me)
        End Sub

        Public Overrides Function GetTypeHashCode() As Integer
            Return 408883412
        End Function

        Private Sub __RenderTree(ByVal __output As System.Web.UI.HtmlTextWriter, ByVal parameterContainer As System.Web.UI.Control)

        End Sub
    End Class
End Namespace
