
Class EventMember5

    Private WithEvents var As System.AppDomain

    Sub EventHandler(ByVal sender As Object, ByVal e As System.AssemblyLoadEventArgs) Handles var.AssemblyLoad

    End Sub
End Class