Option Strict On

Class Nullable4
    Shared Sub Main()
        Dim bool As Boolean?
        Dim b As Byte?
        Dim sb As SByte?
        Dim s As Short?
        Dim us As UShort?
        Dim i As Integer?
        Dim ui As UInteger?
        Dim l As Long?
        Dim ul As Long?
        Dim dec As Decimal?
        Dim f As Single?
        Dim r As Double?
        Dim dt As Date?
        Dim c As Char?

        bool = b

        s = b
        s = CByte(1)



    End Sub
End Class