Class XorConstantExpression1
	Const const1 As Boolean = True Xor False
	Const const2 As Boolean = True Xor True
	Const const3 As Boolean = False Xor True
	Const const4 As Boolean = False Xor False
End Class
