Class test3
    Private Shared Sub Main()
        'system.console.writeline(True)
        ''console.writeline(CByte(1))
        ''system.console.writeline("d"c)
        'a(#1/1/2004#)
        ''system.console.writeline(1D)
        ''system.console.writeline(2.0R)
        ''system.console.writeline(3I)
        ''system.console.writeline(4L)
        '''console.writeline(
        '''console.writeline(CSByte(1))
        '''system.console.writeline(6S)
        '''system.console.writeline(7.0F)
        ''system.console.writeline("8")
        'system.console.writeline(255UI)
        'system.console.writeline(129UI)
        'system.console.writeline(128UI)
        'system.console.writeline(3123456789UI)
        'system.console.writeline(10UL)
        ''system.console.writeline(11US)
        'system.console.writeline(0.0002221128621458452147852145D)
        v("#2/2/2003#")
        r("#1/1/2004#")
    End Sub
    Shared Sub v(ByVal b As String)

    End Sub
    Shared Sub r(ByRef b As String)

    End Sub
End Class