'Author:
'   V. Sudharsan (vsudharsan@novell.com)
'
' (C) 2005 Novell, Inc.

Imports System

Module Test
    Structure AB
        Public Const i As Integer = 10
        Public j As Integer
    End Structure

    Public Function Main() As Integer
    End Function
End Module

