Class OrElseConstantExpression1
	Const const1 As Boolean = True OrElse False
	Const const2 As Boolean = True OrElse True
	Const const3 As Boolean = False OrElse True
	Const const4 As Boolean = False OrElse False
End Class
