Class T
    Public ReadOnly Property Paths(ByVal LanguageCode As String, ByVal parserIsSynchronised As Boolean, Optional ByVal Logical As Boolean = False) As String()
        Get
        End Get
    End Property
    Shared Sub Main()

    End Sub
End Class
