Option Strict On
Class C
    Sub M()
        'Dim dec As Decimal
        'Dim dbl As Double
        'If dec > dbl Then

        'End If
        Dim db As dbnull
        ' N(db, db, l:=2)
        Z("")
    End Sub
    Sub N(ByVal i As Long, ByVal l As Long)

    End Sub
    Sub N(ByVal i As Integer, ByVal l As Long, ByVal ll As Long)
        'Dim dt1, dt2, dt3 As Date
        'dt1 = dt2 + dt3
    End Sub
    Sub N(ByVal i As Integer, ByVal l As Long, ByVal ll As Integer)
        'Dim dt1, dt2, dt3 As Date
        'dt1 = dt2 + dt3
    End Sub
    Sub N(ByVal ParamArray str() As String)

    End Sub
    Sub Z()

    End Sub
End Class