Class Enum1
    Shared Function Main() As Integer
        Dim v As System.Globalization.UnicodeCategory
        v = System.Globalization.UnicodeCategory.Format 'Enum class defines a Format method.
        Return 0
    End Function
End Class