Class OrConstantExpression1
	Const const1 As Boolean = True Or False
	Const const2 As Boolean = True Or True
	Const const3 As Boolean = False Or True
	Const const4 As Boolean = False Or False
End Class
