Option Strict On
Class C
    Sub Z()
        'Dim dec As Decimal
        'Dim dbl As Double
        'If dec > dbl Then

        'End If
        Dim db As dbnull
        ' N(db, db, l:=2)
        M(str:=Nothing)
    End Sub

    Sub M(ByVal ParamArray str() As String)

    End Sub
End Class