Class TypeConversionIntrinsic
    Sub LocalVariables
        Dim  i_bool As Boolean,  i_byte As Byte,  i_sbyte As SByte,  i_short As Short,  i_ushort As UShort,  i_int As Integer,  i_uint as UInteger,  i_long As Long,  i_ulong As ULong,  i_dec As Decimal,  i_sng As Single, i_dbl As Double,  i_chr As Char,  i_str As String,  i_dt As Date,  i_dbnull As DBNull,  i_obj As Object
        Dim  o_bool As Boolean,  o_byte As Byte,  o_sbyte As SByte,  o_short As Short,  o_ushort As UShort,  o_int As Integer,  o_uint as UInteger,  o_long As Long,  o_ulong As ULong,  o_dec As Decimal,  o_sng As Single, o_dbl As Double,  o_chr As Char,  o_str As String,  o_dt As Date,  o_dbnull As DBNull,  o_obj As Object
#If Not STRICT Or ERRORS
        i_ulong = o_bool
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_byte
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_sbyte
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_short
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_ushort
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_int
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_uint
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_long
#End If
        i_ulong = o_ulong
#If Not STRICT Or ERRORS
        i_ulong = o_dec
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_sng
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_dbl
#End If
#If ERRORS
        i_ulong = o_chr
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_str
#End If
#If ERRORS
        i_ulong = o_dt
#End If
#If ERRORS
        i_ulong = o_dbnull
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_obj
#End If
    End Sub
    Sub InLocalOutRef(ByRef o_bool As Boolean, ByRef o_byte As Byte, ByRef o_sbyte As SByte, ByRef o_short As Short, ByRef o_ushort As UShort, ByRef o_int As Integer, ByRef o_uint as UInteger, ByRef o_long As Long, ByRef o_ulong As ULong, ByRef o_dec As Decimal, ByRef o_sng As Single,ByRef o_dbl As Double, ByRef o_chr As Char, ByRef o_str As String, ByRef o_dt As Date, ByRef o_dbnull As DBNull, ByRef o_obj As Object)
        Dim  i_bool As Boolean,  i_byte As Byte,  i_sbyte As SByte,  i_short As Short,  i_ushort As UShort,  i_int As Integer,  i_uint as UInteger,  i_long As Long,  i_ulong As ULong,  i_dec As Decimal,  i_sng As Single, i_dbl As Double,  i_chr As Char,  i_str As String,  i_dt As Date,  i_dbnull As DBNull,  i_obj As Object
#If Not STRICT Or ERRORS
        i_ulong = o_bool
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_byte
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_sbyte
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_short
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_ushort
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_int
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_uint
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_long
#End If
        i_ulong = o_ulong
#If Not STRICT Or ERRORS
        i_ulong = o_dec
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_sng
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_dbl
#End If
#If ERRORS
        i_ulong = o_chr
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_str
#End If
#If ERRORS
        i_ulong = o_dt
#End If
#If ERRORS
        i_ulong = o_dbnull
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_obj
#End If
    End Sub
    Sub InRefOutLocal(ByRef i_bool As Boolean, ByRef i_byte As Byte, ByRef i_sbyte As SByte, ByRef i_short As Short, ByRef i_ushort As UShort, ByRef i_int As Integer, ByRef i_uint as UInteger, ByRef i_long As Long, ByRef i_ulong As ULong, ByRef i_dec As Decimal, ByRef i_sng As Single,ByRef i_dbl As Double, ByRef i_chr As Char, ByRef i_str As String, ByRef i_dt As Date, ByRef i_dbnull As DBNull, ByRef i_obj As Object)
        Dim  o_bool As Boolean,  o_byte As Byte,  o_sbyte As SByte,  o_short As Short,  o_ushort As UShort,  o_int As Integer,  o_uint as UInteger,  o_long As Long,  o_ulong As ULong,  o_dec As Decimal,  o_sng As Single, o_dbl As Double,  o_chr As Char,  o_str As String,  o_dt As Date,  o_dbnull As DBNull,  o_obj As Object
#If Not STRICT Or ERRORS
        i_ulong = o_bool
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_byte
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_sbyte
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_short
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_ushort
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_int
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_uint
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_long
#End If
        i_ulong = o_ulong
#If Not STRICT Or ERRORS
        i_ulong = o_dec
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_sng
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_dbl
#End If
#If ERRORS
        i_ulong = o_chr
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_str
#End If
#If ERRORS
        i_ulong = o_dt
#End If
#If ERRORS
        i_ulong = o_dbnull
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_obj
#End If
    End Sub
    Sub InRefOutRef(ByRef i_bool As Boolean, ByRef i_byte As Byte, ByRef i_sbyte As SByte, ByRef i_short As Short, ByRef i_ushort As UShort, ByRef i_int As Integer, ByRef i_uint as UInteger, ByRef i_long As Long, ByRef i_ulong As ULong, ByRef i_dec As Decimal, ByRef i_sng As Single,ByRef i_dbl As Double, ByRef i_chr As Char, ByRef i_str As String, ByRef i_dt As Date, ByRef i_dbnull As DBNull, ByRef i_obj As Object, ByRef o_bool As Boolean, ByRef o_byte As Byte, ByRef o_sbyte As SByte, ByRef o_short As Short, ByRef o_ushort As UShort, ByRef o_int As Integer, ByRef o_uint as UInteger, ByRef o_long As Long, ByRef o_ulong As ULong, ByRef o_dec As Decimal, ByRef o_sng As Single,ByRef o_dbl As Double, ByRef o_chr As Char, ByRef o_str As String, ByRef o_dt As Date, ByRef o_dbnull As DBNull, ByRef o_obj As Object)
#If Not STRICT Or ERRORS
        i_ulong = o_bool
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_byte
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_sbyte
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_short
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_ushort
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_int
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_uint
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_long
#End If
        i_ulong = o_ulong
#If Not STRICT Or ERRORS
        i_ulong = o_dec
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_sng
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_dbl
#End If
#If ERRORS
        i_ulong = o_chr
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_str
#End If
#If ERRORS
        i_ulong = o_dt
#End If
#If ERRORS
        i_ulong = o_dbnull
#End If
#If Not STRICT Or ERRORS
        i_ulong = o_obj
#End If
    End Sub
End Class
