Imports System
Imports System.Collections
Imports System.Reflection

Namespace ConstructorInitialization6
    Module Test
        Public Value As Integer = 2
        Function Main() As Integer
            Return Value - 2
        End Function
    End Module
End Namespace