Imports System

Class bug_444811
    Declare Ansi Function GetClassName Lib "user32" Alias "GetClassNameA" (ByVal ByValhWnd As IntPtr, ByVal lpClassName As String, ByVal nMaxCount As Int32) As Int32

    Shared Sub Main()

    End Sub
End Class
