Module ConstantExpression7
	Const Exp1 As Integer = microsoft.visualbasic.asc("C"c)
	Const Exp2 As Integer = microsoft.visualbasic.asc("C")
	Const Exp3 As Integer = microsoft.visualbasic.ascw("C"c)
	Const Exp4 As Integer = microsoft.visualbasic.ascw("C")
	Const Exp5 As Char = microsoft.visualbasic.chr(123)
	Const Exp6 As Char = microsoft.visualbasic.chrw(1234)
End Module