Option Strict Off
Imports System
#Region "RegionDirectives"
Module RegionDirectives
    Function Main() As Integer

    End Function

#Region "S"
    Function S()

    End Function
#End Region
End Module
#End Region

