Imports System
Imports System.Collections
Imports System.Reflection

Namespace Optional1
    Class Test
        Shared Sub Test(Optional ByVal v As Long = 2)

        End Sub
        Shared Function Main() As Integer

        End Function
    End Class
End Namespace