Class Attributes7
    <VBFixedArray(CSByte(1))> _
    Dim sb As Integer
    <VBFixedArray(CByte(1))> _
    Dim b As Integer
    <VBFixedArray(CShort(1))> _
    Dim s As Integer
    <VBFixedArray(CUShort(1))> _
    Dim us As Integer
    <VBFixedArray(CInt(1))> _
    Dim i As Integer
    <VBFixedArray(CUInt(1))> _
    Dim ui As Integer
    <VBFixedArray(CLng(1))> _
    Dim l As Integer
    <VBFixedArray(CULng(1))> _
    Dim ul As Integer
End Class