Class BinaryOperatorIs
    Inherits IntrinsicOperatorTests
    Shared Function Main As Integer
        Try
#If Boolean_ERRORS
            M(bool Is bool, "bool Is bool")
#End If
#If Boolean_ERRORS
            M(bool Is b, "bool Is b")
#End If
#If Boolean_ERRORS
            M(bool Is sb, "bool Is sb")
#End If
#If Boolean_ERRORS
            M(bool Is s, "bool Is s")
#End If
#If Boolean_ERRORS
            M(bool Is us, "bool Is us")
#End If
#If Boolean_ERRORS
            M(bool Is i, "bool Is i")
#End If
#If Boolean_ERRORS
            M(bool Is ui, "bool Is ui")
#End If
#If Boolean_ERRORS
            M(bool Is l, "bool Is l")
#End If
#If Boolean_ERRORS
            M(bool Is ul, "bool Is ul")
#End If
#If Boolean_ERRORS
            M(bool Is dec, "bool Is dec")
#End If
#If Boolean_ERRORS
            M(bool Is sng, "bool Is sng")
#End If
#If Boolean_ERRORS
            M(bool Is dbl, "bool Is dbl")
#End If
#If Boolean_ERRORS
            M(bool Is chr, "bool Is chr")
#End If
#If Boolean_ERRORS
            M(bool Is str, "bool Is str")
#End If
#If Boolean_ERRORS
            M(bool Is dt, "bool Is dt")
#End If
#If Boolean_ERRORS
            M(bool Is dbnull, "bool Is dbnull")
#End If
#If Boolean_ERRORS
            M(bool Is obj, "bool Is obj")
#End If
#If Byte_ERRORS
            M(b Is bool, "b Is bool")
#End If
#If Byte_ERRORS
            M(b Is b, "b Is b")
#End If
#If Byte_ERRORS
            M(b Is sb, "b Is sb")
#End If
#If Byte_ERRORS
            M(b Is s, "b Is s")
#End If
#If Byte_ERRORS
            M(b Is us, "b Is us")
#End If
#If Byte_ERRORS
            M(b Is i, "b Is i")
#End If
#If Byte_ERRORS
            M(b Is ui, "b Is ui")
#End If
#If Byte_ERRORS
            M(b Is l, "b Is l")
#End If
#If Byte_ERRORS
            M(b Is ul, "b Is ul")
#End If
#If Byte_ERRORS
            M(b Is dec, "b Is dec")
#End If
#If Byte_ERRORS
            M(b Is sng, "b Is sng")
#End If
#If Byte_ERRORS
            M(b Is dbl, "b Is dbl")
#End If
#If Byte_ERRORS
            M(b Is chr, "b Is chr")
#End If
#If Byte_ERRORS
            M(b Is str, "b Is str")
#End If
#If Byte_ERRORS
            M(b Is dt, "b Is dt")
#End If
#If Byte_ERRORS
            M(b Is dbnull, "b Is dbnull")
#End If
#If Byte_ERRORS
            M(b Is obj, "b Is obj")
#End If
#If SByte_ERRORS
            M(sb Is bool, "sb Is bool")
#End If
#If SByte_ERRORS
            M(sb Is b, "sb Is b")
#End If
#If SByte_ERRORS
            M(sb Is sb, "sb Is sb")
#End If
#If SByte_ERRORS
            M(sb Is s, "sb Is s")
#End If
#If SByte_ERRORS
            M(sb Is us, "sb Is us")
#End If
#If SByte_ERRORS
            M(sb Is i, "sb Is i")
#End If
#If SByte_ERRORS
            M(sb Is ui, "sb Is ui")
#End If
#If SByte_ERRORS
            M(sb Is l, "sb Is l")
#End If
#If SByte_ERRORS
            M(sb Is ul, "sb Is ul")
#End If
#If SByte_ERRORS
            M(sb Is dec, "sb Is dec")
#End If
#If SByte_ERRORS
            M(sb Is sng, "sb Is sng")
#End If
#If SByte_ERRORS
            M(sb Is dbl, "sb Is dbl")
#End If
#If SByte_ERRORS
            M(sb Is chr, "sb Is chr")
#End If
#If SByte_ERRORS
            M(sb Is str, "sb Is str")
#End If
#If SByte_ERRORS
            M(sb Is dt, "sb Is dt")
#End If
#If SByte_ERRORS
            M(sb Is dbnull, "sb Is dbnull")
#End If
#If SByte_ERRORS
            M(sb Is obj, "sb Is obj")
#End If
#If Int16_ERRORS
            M(s Is bool, "s Is bool")
#End If
#If Int16_ERRORS
            M(s Is b, "s Is b")
#End If
#If Int16_ERRORS
            M(s Is sb, "s Is sb")
#End If
#If Int16_ERRORS
            M(s Is s, "s Is s")
#End If
#If Int16_ERRORS
            M(s Is us, "s Is us")
#End If
#If Int16_ERRORS
            M(s Is i, "s Is i")
#End If
#If Int16_ERRORS
            M(s Is ui, "s Is ui")
#End If
#If Int16_ERRORS
            M(s Is l, "s Is l")
#End If
#If Int16_ERRORS
            M(s Is ul, "s Is ul")
#End If
#If Int16_ERRORS
            M(s Is dec, "s Is dec")
#End If
#If Int16_ERRORS
            M(s Is sng, "s Is sng")
#End If
#If Int16_ERRORS
            M(s Is dbl, "s Is dbl")
#End If
#If Int16_ERRORS
            M(s Is chr, "s Is chr")
#End If
#If Int16_ERRORS
            M(s Is str, "s Is str")
#End If
#If Int16_ERRORS
            M(s Is dt, "s Is dt")
#End If
#If Int16_ERRORS
            M(s Is dbnull, "s Is dbnull")
#End If
#If Int16_ERRORS
            M(s Is obj, "s Is obj")
#End If
#If UInt16_ERRORS
            M(us Is bool, "us Is bool")
#End If
#If UInt16_ERRORS
            M(us Is b, "us Is b")
#End If
#If UInt16_ERRORS
            M(us Is sb, "us Is sb")
#End If
#If UInt16_ERRORS
            M(us Is s, "us Is s")
#End If
#If UInt16_ERRORS
            M(us Is us, "us Is us")
#End If
#If UInt16_ERRORS
            M(us Is i, "us Is i")
#End If
#If UInt16_ERRORS
            M(us Is ui, "us Is ui")
#End If
#If UInt16_ERRORS
            M(us Is l, "us Is l")
#End If
#If UInt16_ERRORS
            M(us Is ul, "us Is ul")
#End If
#If UInt16_ERRORS
            M(us Is dec, "us Is dec")
#End If
#If UInt16_ERRORS
            M(us Is sng, "us Is sng")
#End If
#If UInt16_ERRORS
            M(us Is dbl, "us Is dbl")
#End If
#If UInt16_ERRORS
            M(us Is chr, "us Is chr")
#End If
#If UInt16_ERRORS
            M(us Is str, "us Is str")
#End If
#If UInt16_ERRORS
            M(us Is dt, "us Is dt")
#End If
#If UInt16_ERRORS
            M(us Is dbnull, "us Is dbnull")
#End If
#If UInt16_ERRORS
            M(us Is obj, "us Is obj")
#End If
#If Int32_ERRORS
            M(i Is bool, "i Is bool")
#End If
#If Int32_ERRORS
            M(i Is b, "i Is b")
#End If
#If Int32_ERRORS
            M(i Is sb, "i Is sb")
#End If
#If Int32_ERRORS
            M(i Is s, "i Is s")
#End If
#If Int32_ERRORS
            M(i Is us, "i Is us")
#End If
#If Int32_ERRORS
            M(i Is i, "i Is i")
#End If
#If Int32_ERRORS
            M(i Is ui, "i Is ui")
#End If
#If Int32_ERRORS
            M(i Is l, "i Is l")
#End If
#If Int32_ERRORS
            M(i Is ul, "i Is ul")
#End If
#If Int32_ERRORS
            M(i Is dec, "i Is dec")
#End If
#If Int32_ERRORS
            M(i Is sng, "i Is sng")
#End If
#If Int32_ERRORS
            M(i Is dbl, "i Is dbl")
#End If
#If Int32_ERRORS
            M(i Is chr, "i Is chr")
#End If
#If Int32_ERRORS
            M(i Is str, "i Is str")
#End If
#If Int32_ERRORS
            M(i Is dt, "i Is dt")
#End If
#If Int32_ERRORS
            M(i Is dbnull, "i Is dbnull")
#End If
#If Int32_ERRORS
            M(i Is obj, "i Is obj")
#End If
#If UInt32_ERRORS
            M(ui Is bool, "ui Is bool")
#End If
#If UInt32_ERRORS
            M(ui Is b, "ui Is b")
#End If
#If UInt32_ERRORS
            M(ui Is sb, "ui Is sb")
#End If
#If UInt32_ERRORS
            M(ui Is s, "ui Is s")
#End If
#If UInt32_ERRORS
            M(ui Is us, "ui Is us")
#End If
#If UInt32_ERRORS
            M(ui Is i, "ui Is i")
#End If
#If UInt32_ERRORS
            M(ui Is ui, "ui Is ui")
#End If
#If UInt32_ERRORS
            M(ui Is l, "ui Is l")
#End If
#If UInt32_ERRORS
            M(ui Is ul, "ui Is ul")
#End If
#If UInt32_ERRORS
            M(ui Is dec, "ui Is dec")
#End If
#If UInt32_ERRORS
            M(ui Is sng, "ui Is sng")
#End If
#If UInt32_ERRORS
            M(ui Is dbl, "ui Is dbl")
#End If
#If UInt32_ERRORS
            M(ui Is chr, "ui Is chr")
#End If
#If UInt32_ERRORS
            M(ui Is str, "ui Is str")
#End If
#If UInt32_ERRORS
            M(ui Is dt, "ui Is dt")
#End If
#If UInt32_ERRORS
            M(ui Is dbnull, "ui Is dbnull")
#End If
#If UInt32_ERRORS
            M(ui Is obj, "ui Is obj")
#End If
#If Int64_ERRORS
            M(l Is bool, "l Is bool")
#End If
#If Int64_ERRORS
            M(l Is b, "l Is b")
#End If
#If Int64_ERRORS
            M(l Is sb, "l Is sb")
#End If
#If Int64_ERRORS
            M(l Is s, "l Is s")
#End If
#If Int64_ERRORS
            M(l Is us, "l Is us")
#End If
#If Int64_ERRORS
            M(l Is i, "l Is i")
#End If
#If Int64_ERRORS
            M(l Is ui, "l Is ui")
#End If
#If Int64_ERRORS
            M(l Is l, "l Is l")
#End If
#If Int64_ERRORS
            M(l Is ul, "l Is ul")
#End If
#If Int64_ERRORS
            M(l Is dec, "l Is dec")
#End If
#If Int64_ERRORS
            M(l Is sng, "l Is sng")
#End If
#If Int64_ERRORS
            M(l Is dbl, "l Is dbl")
#End If
#If Int64_ERRORS
            M(l Is chr, "l Is chr")
#End If
#If Int64_ERRORS
            M(l Is str, "l Is str")
#End If
#If Int64_ERRORS
            M(l Is dt, "l Is dt")
#End If
#If Int64_ERRORS
            M(l Is dbnull, "l Is dbnull")
#End If
#If Int64_ERRORS
            M(l Is obj, "l Is obj")
#End If
#If UInt64_ERRORS
            M(ul Is bool, "ul Is bool")
#End If
#If UInt64_ERRORS
            M(ul Is b, "ul Is b")
#End If
#If UInt64_ERRORS
            M(ul Is sb, "ul Is sb")
#End If
#If UInt64_ERRORS
            M(ul Is s, "ul Is s")
#End If
#If UInt64_ERRORS
            M(ul Is us, "ul Is us")
#End If
#If UInt64_ERRORS
            M(ul Is i, "ul Is i")
#End If
#If UInt64_ERRORS
            M(ul Is ui, "ul Is ui")
#End If
#If UInt64_ERRORS
            M(ul Is l, "ul Is l")
#End If
#If UInt64_ERRORS
            M(ul Is ul, "ul Is ul")
#End If
#If UInt64_ERRORS
            M(ul Is dec, "ul Is dec")
#End If
#If UInt64_ERRORS
            M(ul Is sng, "ul Is sng")
#End If
#If UInt64_ERRORS
            M(ul Is dbl, "ul Is dbl")
#End If
#If UInt64_ERRORS
            M(ul Is chr, "ul Is chr")
#End If
#If UInt64_ERRORS
            M(ul Is str, "ul Is str")
#End If
#If UInt64_ERRORS
            M(ul Is dt, "ul Is dt")
#End If
#If UInt64_ERRORS
            M(ul Is dbnull, "ul Is dbnull")
#End If
#If UInt64_ERRORS
            M(ul Is obj, "ul Is obj")
#End If
#If Decimal_ERRORS
            M(dec Is bool, "dec Is bool")
#End If
#If Decimal_ERRORS
            M(dec Is b, "dec Is b")
#End If
#If Decimal_ERRORS
            M(dec Is sb, "dec Is sb")
#End If
#If Decimal_ERRORS
            M(dec Is s, "dec Is s")
#End If
#If Decimal_ERRORS
            M(dec Is us, "dec Is us")
#End If
#If Decimal_ERRORS
            M(dec Is i, "dec Is i")
#End If
#If Decimal_ERRORS
            M(dec Is ui, "dec Is ui")
#End If
#If Decimal_ERRORS
            M(dec Is l, "dec Is l")
#End If
#If Decimal_ERRORS
            M(dec Is ul, "dec Is ul")
#End If
#If Decimal_ERRORS
            M(dec Is dec, "dec Is dec")
#End If
#If Decimal_ERRORS
            M(dec Is sng, "dec Is sng")
#End If
#If Decimal_ERRORS
            M(dec Is dbl, "dec Is dbl")
#End If
#If Decimal_ERRORS
            M(dec Is chr, "dec Is chr")
#End If
#If Decimal_ERRORS
            M(dec Is str, "dec Is str")
#End If
#If Decimal_ERRORS
            M(dec Is dt, "dec Is dt")
#End If
#If Decimal_ERRORS
            M(dec Is dbnull, "dec Is dbnull")
#End If
#If Decimal_ERRORS
            M(dec Is obj, "dec Is obj")
#End If
#If Single_ERRORS
            M(sng Is bool, "sng Is bool")
#End If
#If Single_ERRORS
            M(sng Is b, "sng Is b")
#End If
#If Single_ERRORS
            M(sng Is sb, "sng Is sb")
#End If
#If Single_ERRORS
            M(sng Is s, "sng Is s")
#End If
#If Single_ERRORS
            M(sng Is us, "sng Is us")
#End If
#If Single_ERRORS
            M(sng Is i, "sng Is i")
#End If
#If Single_ERRORS
            M(sng Is ui, "sng Is ui")
#End If
#If Single_ERRORS
            M(sng Is l, "sng Is l")
#End If
#If Single_ERRORS
            M(sng Is ul, "sng Is ul")
#End If
#If Single_ERRORS
            M(sng Is dec, "sng Is dec")
#End If
#If Single_ERRORS
            M(sng Is sng, "sng Is sng")
#End If
#If Single_ERRORS
            M(sng Is dbl, "sng Is dbl")
#End If
#If Single_ERRORS
            M(sng Is chr, "sng Is chr")
#End If
#If Single_ERRORS
            M(sng Is str, "sng Is str")
#End If
#If Single_ERRORS
            M(sng Is dt, "sng Is dt")
#End If
#If Single_ERRORS
            M(sng Is dbnull, "sng Is dbnull")
#End If
#If Single_ERRORS
            M(sng Is obj, "sng Is obj")
#End If
#If Double_ERRORS
            M(dbl Is bool, "dbl Is bool")
#End If
#If Double_ERRORS
            M(dbl Is b, "dbl Is b")
#End If
#If Double_ERRORS
            M(dbl Is sb, "dbl Is sb")
#End If
#If Double_ERRORS
            M(dbl Is s, "dbl Is s")
#End If
#If Double_ERRORS
            M(dbl Is us, "dbl Is us")
#End If
#If Double_ERRORS
            M(dbl Is i, "dbl Is i")
#End If
#If Double_ERRORS
            M(dbl Is ui, "dbl Is ui")
#End If
#If Double_ERRORS
            M(dbl Is l, "dbl Is l")
#End If
#If Double_ERRORS
            M(dbl Is ul, "dbl Is ul")
#End If
#If Double_ERRORS
            M(dbl Is dec, "dbl Is dec")
#End If
#If Double_ERRORS
            M(dbl Is sng, "dbl Is sng")
#End If
#If Double_ERRORS
            M(dbl Is dbl, "dbl Is dbl")
#End If
#If Double_ERRORS
            M(dbl Is chr, "dbl Is chr")
#End If
#If Double_ERRORS
            M(dbl Is str, "dbl Is str")
#End If
#If Double_ERRORS
            M(dbl Is dt, "dbl Is dt")
#End If
#If Double_ERRORS
            M(dbl Is dbnull, "dbl Is dbnull")
#End If
#If Double_ERRORS
            M(dbl Is obj, "dbl Is obj")
#End If
#If Char_ERRORS
            M(chr Is bool, "chr Is bool")
#End If
#If Char_ERRORS
            M(chr Is b, "chr Is b")
#End If
#If Char_ERRORS
            M(chr Is sb, "chr Is sb")
#End If
#If Char_ERRORS
            M(chr Is s, "chr Is s")
#End If
#If Char_ERRORS
            M(chr Is us, "chr Is us")
#End If
#If Char_ERRORS
            M(chr Is i, "chr Is i")
#End If
#If Char_ERRORS
            M(chr Is ui, "chr Is ui")
#End If
#If Char_ERRORS
            M(chr Is l, "chr Is l")
#End If
#If Char_ERRORS
            M(chr Is ul, "chr Is ul")
#End If
#If Char_ERRORS
            M(chr Is dec, "chr Is dec")
#End If
#If Char_ERRORS
            M(chr Is sng, "chr Is sng")
#End If
#If Char_ERRORS
            M(chr Is dbl, "chr Is dbl")
#End If
#If Char_ERRORS
            M(chr Is chr, "chr Is chr")
#End If
#If Char_ERRORS
            M(chr Is str, "chr Is str")
#End If
#If Char_ERRORS
            M(chr Is dt, "chr Is dt")
#End If
#If Char_ERRORS
            M(chr Is dbnull, "chr Is dbnull")
#End If
#If Char_ERRORS
            M(chr Is obj, "chr Is obj")
#End If
#If String_ERRORS
            M(str Is bool, "str Is bool")
#End If
#If String_ERRORS
            M(str Is b, "str Is b")
#End If
#If String_ERRORS
            M(str Is sb, "str Is sb")
#End If
#If String_ERRORS
            M(str Is s, "str Is s")
#End If
#If String_ERRORS
            M(str Is us, "str Is us")
#End If
#If String_ERRORS
            M(str Is i, "str Is i")
#End If
#If String_ERRORS
            M(str Is ui, "str Is ui")
#End If
#If String_ERRORS
            M(str Is l, "str Is l")
#End If
#If String_ERRORS
            M(str Is ul, "str Is ul")
#End If
#If String_ERRORS
            M(str Is dec, "str Is dec")
#End If
#If String_ERRORS
            M(str Is sng, "str Is sng")
#End If
#If String_ERRORS
            M(str Is dbl, "str Is dbl")
#End If
#If String_ERRORS
            M(str Is chr, "str Is chr")
#End If
            expected_tc = TypeCode.Boolean
            M(str Is str, "str Is str")
#If String_ERRORS
            M(str Is dt, "str Is dt")
#End If
#If String_ERRORS
            M(str Is dbnull, "str Is dbnull")
#End If
            expected_tc = TypeCode.Boolean
            M(str Is obj, "str Is obj")
#If DateTime_ERRORS
            M(dt Is bool, "dt Is bool")
#End If
#If DateTime_ERRORS
            M(dt Is b, "dt Is b")
#End If
#If DateTime_ERRORS
            M(dt Is sb, "dt Is sb")
#End If
#If DateTime_ERRORS
            M(dt Is s, "dt Is s")
#End If
#If DateTime_ERRORS
            M(dt Is us, "dt Is us")
#End If
#If DateTime_ERRORS
            M(dt Is i, "dt Is i")
#End If
#If DateTime_ERRORS
            M(dt Is ui, "dt Is ui")
#End If
#If DateTime_ERRORS
            M(dt Is l, "dt Is l")
#End If
#If DateTime_ERRORS
            M(dt Is ul, "dt Is ul")
#End If
#If DateTime_ERRORS
            M(dt Is dec, "dt Is dec")
#End If
#If DateTime_ERRORS
            M(dt Is sng, "dt Is sng")
#End If
#If DateTime_ERRORS
            M(dt Is dbl, "dt Is dbl")
#End If
#If DateTime_ERRORS
            M(dt Is chr, "dt Is chr")
#End If
#If DateTime_ERRORS
            M(dt Is str, "dt Is str")
#End If
#If DateTime_ERRORS
            M(dt Is dt, "dt Is dt")
#End If
#If DateTime_ERRORS
            M(dt Is dbnull, "dt Is dbnull")
#End If
#If DateTime_ERRORS
            M(dt Is obj, "dt Is obj")
#End If
#If DBNull_ERRORS
            M(dbnull Is bool, "dbnull Is bool")
#End If
#If DBNull_ERRORS
            M(dbnull Is b, "dbnull Is b")
#End If
#If DBNull_ERRORS
            M(dbnull Is sb, "dbnull Is sb")
#End If
#If DBNull_ERRORS
            M(dbnull Is s, "dbnull Is s")
#End If
#If DBNull_ERRORS
            M(dbnull Is us, "dbnull Is us")
#End If
#If DBNull_ERRORS
            M(dbnull Is i, "dbnull Is i")
#End If
#If DBNull_ERRORS
            M(dbnull Is ui, "dbnull Is ui")
#End If
#If DBNull_ERRORS
            M(dbnull Is l, "dbnull Is l")
#End If
#If DBNull_ERRORS
            M(dbnull Is ul, "dbnull Is ul")
#End If
#If DBNull_ERRORS
            M(dbnull Is dec, "dbnull Is dec")
#End If
#If DBNull_ERRORS
            M(dbnull Is sng, "dbnull Is sng")
#End If
#If DBNull_ERRORS
            M(dbnull Is dbl, "dbnull Is dbl")
#End If
#If DBNull_ERRORS
            M(dbnull Is chr, "dbnull Is chr")
#End If
#If DBNull_ERRORS
            M(dbnull Is str, "dbnull Is str")
#End If
#If DBNull_ERRORS
            M(dbnull Is dt, "dbnull Is dt")
#End If
#If DBNull_ERRORS
            M(dbnull Is dbnull, "dbnull Is dbnull")
#End If
#If DBNull_ERRORS
            M(dbnull Is obj, "dbnull Is obj")
#End If
#If Object_ERRORS
            M(obj Is bool, "obj Is bool")
#End If
#If Object_ERRORS
            M(obj Is b, "obj Is b")
#End If
#If Object_ERRORS
            M(obj Is sb, "obj Is sb")
#End If
#If Object_ERRORS
            M(obj Is s, "obj Is s")
#End If
#If Object_ERRORS
            M(obj Is us, "obj Is us")
#End If
#If Object_ERRORS
            M(obj Is i, "obj Is i")
#End If
#If Object_ERRORS
            M(obj Is ui, "obj Is ui")
#End If
#If Object_ERRORS
            M(obj Is l, "obj Is l")
#End If
#If Object_ERRORS
            M(obj Is ul, "obj Is ul")
#End If
#If Object_ERRORS
            M(obj Is dec, "obj Is dec")
#End If
#If Object_ERRORS
            M(obj Is sng, "obj Is sng")
#End If
#If Object_ERRORS
            M(obj Is dbl, "obj Is dbl")
#End If
#If Object_ERRORS
            M(obj Is chr, "obj Is chr")
#End If
            expected_tc = TypeCode.Boolean
            M(obj Is str, "obj Is str")
#If Object_ERRORS
            M(obj Is dt, "obj Is dt")
#End If
#If Object_ERRORS
            M(obj Is dbnull, "obj Is dbnull")
#End If
            expected_tc = TypeCode.Boolean
            M(obj Is obj, "obj Is obj")
        If failures > 0 Then Return 1
    Catch ex As Exception
        Console.WriteLine ("Exception: {0}", ex)
        Return 2
    End Try
    Return 0
    End Function
End Class
