'Author:
'   V. Sudharsan (vsudharsan@novell.com)
'
' (C) 2005 Novell, Inc.

'To Prove Constructors can be overloaded

Imports System

Class A
    Private Shared Sub New()
    End Sub
End Class

Module Test
    Public Sub Main()
    End Sub
End Module

