'Author:
'   V. Sudharsan (vsudharsan@novell.com)
'
' (C) 2005 Novell, Inc.

Module Test
    Function Main() As Integer
        Dim b(4, 1) As Integer

        b(0, 0) = 0 : b(0, 1) = 1
        b(1, 0) = 2 : b(1, 1) = 3
        b(2, 0) = 4 : b(2, 1) = 5
        b(3, 0) = 6 : b(3, 1) = 7
        b(4, 0) = 8 : b(4, 1) = 9
    End Function
End Module
