'Option Strict On
Imports system.windows.forms

Public Class Test
    Inherits Form
    Shared var As Integer = 2

    Private Sub Form1_ResizeEnd(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.ResizeEnd

    End Sub
End Class
