Imports System
Imports System.Collections
Imports System.Reflection

Namespace MethodResolution7
    Class Test
        Shared Function Main() As Integer
            Dim inputParameters()() As Test
            ReDim inputparameters(0)
        End Function
    End Class
End Namespace