class AndAlsoConstantExpression1  
	Const const1 As Boolean = True AndAlso False
	Const const2 As Boolean = True AndAlso True
	Const const3 As Boolean = False AndAlso True
	Const const4 As Boolean = False AndAlso False
end class  
