Imports S = System

Enum E1
    e
End Enum

Enum B1 As Byte
    e
End Enum
Enum SB1 As SByte
    e
End Enum
Enum S1 As Short
    e
End Enum
Enum US1 As UShort
    e
End Enum
Enum I1 As Integer
    e
End Enum
Enum UI1 As UInteger
    e
End Enum
Enum L1 As Long
    e
End Enum
Enum UL1 As ULong
    e
End Enum

Enum B2 As System.Byte
    e
End Enum
Enum SB2 As System.SByte
    e
End Enum
Enum S2 As System.Int16
    e
End Enum
Enum US2 As System.UInt16
    e
End Enum
Enum I2 As System.Int32
    e
End Enum
Enum UI2 As System.UInt32
    e
End Enum
Enum L2 As System.Int64
    e
End Enum
Enum UL2 As System.UInt64
    e
End Enum

Enum B3 As S.Byte
    e
End Enum
Enum SB3 As S.SByte
    e
End Enum
Enum S3 As S.Int16
    e
End Enum
Enum US3 As S.UInt16
    e
End Enum
Enum I3 As S.Int32
    e
End Enum
Enum UI3 As S.UInt32
    e
End Enum
Enum L3 As S.Int64
    e
End Enum
Enum UL3 As S.UInt64
    e
End Enum

