Option Explicit On
Option Strict On

Imports System
Imports System.Web
Imports System.Web.UI


Namespace aspnet

    Public Partial Class Default
        Inherits System.Web.UI.Page
    End Class
End Namespace