Class FunctionReturnVariable1
    Shared Function Main() As Integer
        Test()
    End Function

    Shared Function Test() As String
        Test = ""
        Test = Test.ToString()
        Test.ToString()
    End Function
End Class