Option Strict On
Module A
    Public Sub M()

    End Sub
End Module
Module B
    Public Sub M()

    End Sub
End Module
Module C
    Public Sub M()

    End Sub
End Module
Class D
    Shared Sub Main()
        M()
    End Sub
End Class