Module bug
    Property I() As Integer
        Get

        End Get
        Set(ByVal value As Integer)

        End Set
    End Property
    Sub Main()
        For i = 1 To 2

        Next
    End Sub
End Module