Class C
    Shared Sub Main()
        System.Console.WriteLine(“abc”)
        'System.Console.WriteLine("”")
        System.Console.WriteLine("””")
        System.Console.WriteLine(““"”)
    End Sub
End Class