'A total of 96 functions have been generated.
Option Strict Off
Imports System

Class BinaryAdd_Generated
    '<Test()> Sub Add_Boolean_Boolean()
    '    Dim t1 As Boolean = Nothing, t2 As Boolean = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Boolean_Byte()
    '    Dim t1 As Boolean = Nothing, t2 As Byte = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Boolean_Double()
    '    Dim t1 As Boolean = Nothing, t2 As Double = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Boolean_Int32()
    '    Dim t1 As Boolean = Nothing, t2 As Int32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Boolean_Int64()
    '    Dim t1 As Boolean = Nothing, t2 As Int64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Boolean_Object()
    '    Dim t1 As Boolean = Nothing, t2 As Object = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Boolean_Int16()
    '    Dim t1 As Boolean = Nothing, t2 As Int16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Boolean_Single()
    '    Dim t1 As Boolean = Nothing, t2 As Single = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Boolean_String()
    '    Dim t1 As Boolean = Nothing, t2 As String = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Boolean_SByte()
    '    Dim t1 As Boolean = Nothing, t2 As SByte = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Boolean_UInt16()
    '    Dim t1 As Boolean = Nothing, t2 As UInt16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Boolean_UInt32()
    '    Dim t1 As Boolean = Nothing, t2 As UInt32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Boolean_UInt64()
    '    Dim t1 As Boolean = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    <Test()> Sub Add_Byte_Byte()
        Dim t1 As Byte = Nothing, t2 As Byte = Nothing
        Dim t As Object
        Stop
        t = t1 + t2
        Stop
        '       Assert.AreEqual(t, t1 + t2, "should be equal")
    End Sub

    <Test()> Sub Add_Byte_Double()
        Dim t1 As Byte = Nothing, t2 As Double = Nothing
        Dim t As Object
        Stop
        t = t1 + t2
        Stop
        '       Assert.AreEqual(t, t1 + t2, "should be equal")
    End Sub

    <Test()> Sub Add_Byte_Int32()
        Dim t1 As Byte = Nothing, t2 As Int32 = Nothing
        Dim t As Object
        Stop
        t = t1 + t2
        Stop
        '       Assert.AreEqual(t, t1 + t2, "should be equal")
    End Sub

    <Test()> Sub Add_Byte_Int64()
        Dim t1 As Byte = Nothing, t2 As Int64 = Nothing
        Dim t As Object
        Stop
        t = t1 + t2
        Stop
        '       Assert.AreEqual(t, t1 + t2, "should be equal")
    End Sub

    '<Test()> Sub Add_Byte_Object()
    '    Dim t1 As Byte = Nothing, t2 As Object = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Byte_Int16()
    '    Dim t1 As Byte = Nothing, t2 As Int16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Byte_Single()
    '    Dim t1 As Byte = Nothing, t2 As Single = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Byte_String()
    '    Dim t1 As Byte = Nothing, t2 As String = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Byte_SByte()
    '    Dim t1 As Byte = Nothing, t2 As SByte = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Byte_UInt16()
    '    Dim t1 As Byte = Nothing, t2 As UInt16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Byte_UInt32()
    '    Dim t1 As Byte = Nothing, t2 As UInt32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Byte_UInt64()
    '    Dim t1 As Byte = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Char_Object()
    '    Dim t1 As Char = Nothing, t2 As Object = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Char_String()
    '    Dim t1 As Char = Nothing, t2 As String = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_DateTime_DateTime()
    '    Dim t1 As DateTime = Nothing, t2 As DateTime = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_DateTime_Object()
    '    Dim t1 As DateTime = Nothing, t2 As Object = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_DateTime_String()
    '    Dim t1 As DateTime = Nothing, t2 As String = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Double_Double()
    '    Dim t1 As Double = Nothing, t2 As Double = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Double_Int32()
    '    Dim t1 As Double = Nothing, t2 As Int32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Double_Int64()
    '    Dim t1 As Double = Nothing, t2 As Int64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Double_Object()
    '    Dim t1 As Double = Nothing, t2 As Object = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Double_Int16()
    '    Dim t1 As Double = Nothing, t2 As Int16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Double_Single()
    '    Dim t1 As Double = Nothing, t2 As Single = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Double_String()
    '    Dim t1 As Double = Nothing, t2 As String = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Double_SByte()
    '    Dim t1 As Double = Nothing, t2 As SByte = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Double_UInt16()
    '    Dim t1 As Double = Nothing, t2 As UInt16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Double_UInt32()
    '    Dim t1 As Double = Nothing, t2 As UInt32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Double_UInt64()
    '    Dim t1 As Double = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int32_Int32()
    '    Dim t1 As Int32 = Nothing, t2 As Int32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int32_Int64()
    '    Dim t1 As Int32 = Nothing, t2 As Int64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int32_Object()
    '    Dim t1 As Int32 = Nothing, t2 As Object = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int32_Int16()
    '    Dim t1 As Int32 = Nothing, t2 As Int16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int32_Single()
    '    Dim t1 As Int32 = Nothing, t2 As Single = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int32_String()
    '    Dim t1 As Int32 = Nothing, t2 As String = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int32_SByte()
    '    Dim t1 As Int32 = Nothing, t2 As SByte = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int32_UInt16()
    '    Dim t1 As Int32 = Nothing, t2 As UInt16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int32_UInt32()
    '    Dim t1 As Int32 = Nothing, t2 As UInt32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int32_UInt64()
    '    Dim t1 As Int32 = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int64_Int64()
    '    Dim t1 As Int64 = Nothing, t2 As Int64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int64_Object()
    '    Dim t1 As Int64 = Nothing, t2 As Object = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int64_Int16()
    '    Dim t1 As Int64 = Nothing, t2 As Int16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int64_Single()
    '    Dim t1 As Int64 = Nothing, t2 As Single = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int64_String()
    '    Dim t1 As Int64 = Nothing, t2 As String = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int64_SByte()
    '    Dim t1 As Int64 = Nothing, t2 As SByte = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int64_UInt16()
    '    Dim t1 As Int64 = Nothing, t2 As UInt16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int64_UInt32()
    '    Dim t1 As Int64 = Nothing, t2 As UInt32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int64_UInt64()
    '    Dim t1 As Int64 = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Object_Object()
    '    Dim t1 As Object = Nothing, t2 As Object = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Object_Int16()
    '    Dim t1 As Object = Nothing, t2 As Int16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Object_Single()
    '    Dim t1 As Object = Nothing, t2 As Single = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Object_String()
    '    Dim t1 As Object = Nothing, t2 As String = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Object_SByte()
    '    Dim t1 As Object = Nothing, t2 As SByte = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Object_UInt16()
    '    Dim t1 As Object = Nothing, t2 As UInt16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Object_UInt32()
    '    Dim t1 As Object = Nothing, t2 As UInt32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Object_UInt64()
    '    Dim t1 As Object = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int16_Int16()
    '    Dim t1 As Int16 = Nothing, t2 As Int16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int16_Single()
    '    Dim t1 As Int16 = Nothing, t2 As Single = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int16_String()
    '    Dim t1 As Int16 = Nothing, t2 As String = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int16_SByte()
    '    Dim t1 As Int16 = Nothing, t2 As SByte = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int16_UInt16()
    '    Dim t1 As Int16 = Nothing, t2 As UInt16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int16_UInt32()
    '    Dim t1 As Int16 = Nothing, t2 As UInt32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Int16_UInt64()
    '    Dim t1 As Int16 = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Single_Single()
    '    Dim t1 As Single = Nothing, t2 As Single = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Single_String()
    '    Dim t1 As Single = Nothing, t2 As String = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Single_SByte()
    '    Dim t1 As Single = Nothing, t2 As SByte = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Single_UInt16()
    '    Dim t1 As Single = Nothing, t2 As UInt16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Single_UInt32()
    '    Dim t1 As Single = Nothing, t2 As UInt32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_Single_UInt64()
    '    Dim t1 As Single = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_String_String()
    '    Dim t1 As String = Nothing, t2 As String = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_String_SByte()
    '    Dim t1 As String = Nothing, t2 As SByte = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_String_UInt16()
    '    Dim t1 As String = Nothing, t2 As UInt16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_String_UInt32()
    '    Dim t1 As String = Nothing, t2 As UInt32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_String_UInt64()
    '    Dim t1 As String = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_SByte_SByte()
    '    Dim t1 As SByte = Nothing, t2 As SByte = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_SByte_UInt16()
    '    Dim t1 As SByte = Nothing, t2 As UInt16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_SByte_UInt32()
    '    Dim t1 As SByte = Nothing, t2 As UInt32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_SByte_UInt64()
    '    Dim t1 As SByte = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_UInt16_UInt16()
    '    Dim t1 As UInt16 = Nothing, t2 As UInt16 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_UInt16_UInt32()
    '    Dim t1 As UInt16 = Nothing, t2 As UInt32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_UInt16_UInt64()
    '    Dim t1 As UInt16 = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_UInt32_UInt32()
    '    Dim t1 As UInt32 = Nothing, t2 As UInt32 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_UInt32_UInt64()
    '    Dim t1 As UInt32 = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

    '<Test()> Sub Add_UInt64_UInt64()
    '    Dim t1 As UInt64 = Nothing, t2 As UInt64 = Nothing
    '    Dim t As Object
    '    Stop
    '    t = t1 + t2
    '    Stop
    '    '       Assert.AreEqual(t, t1 + t2, "should be equal")
    'End Sub

End Class
