Imports System
Imports System.Collections
Imports System.Reflection

Namespace Enum4
    Class Test
        Shared Function Main() As Integer
            Dim e As [Enum]
        End Function
    End Class
    Enum En
        value222
    End Enum

End Namespace