Class BinaryOperatorIsNot
    Inherits IntrinsicOperatorTests
    Shared Function Main As Integer
        Try
#If Boolean_ERRORS
            M(bool IsNot bool, "bool IsNot bool")
#End If
#If Boolean_ERRORS
            M(bool IsNot b, "bool IsNot b")
#End If
#If Boolean_ERRORS
            M(bool IsNot sb, "bool IsNot sb")
#End If
#If Boolean_ERRORS
            M(bool IsNot s, "bool IsNot s")
#End If
#If Boolean_ERRORS
            M(bool IsNot us, "bool IsNot us")
#End If
#If Boolean_ERRORS
            M(bool IsNot i, "bool IsNot i")
#End If
#If Boolean_ERRORS
            M(bool IsNot ui, "bool IsNot ui")
#End If
#If Boolean_ERRORS
            M(bool IsNot l, "bool IsNot l")
#End If
#If Boolean_ERRORS
            M(bool IsNot ul, "bool IsNot ul")
#End If
#If Boolean_ERRORS
            M(bool IsNot dec, "bool IsNot dec")
#End If
#If Boolean_ERRORS
            M(bool IsNot sng, "bool IsNot sng")
#End If
#If Boolean_ERRORS
            M(bool IsNot dbl, "bool IsNot dbl")
#End If
#If Boolean_ERRORS
            M(bool IsNot chr, "bool IsNot chr")
#End If
#If Boolean_ERRORS
            M(bool IsNot str, "bool IsNot str")
#End If
#If Boolean_ERRORS
            M(bool IsNot dt, "bool IsNot dt")
#End If
#If Boolean_ERRORS
            M(bool IsNot dbnull, "bool IsNot dbnull")
#End If
#If Boolean_ERRORS
            M(bool IsNot obj, "bool IsNot obj")
#End If
#If Byte_ERRORS
            M(b IsNot bool, "b IsNot bool")
#End If
#If Byte_ERRORS
            M(b IsNot b, "b IsNot b")
#End If
#If Byte_ERRORS
            M(b IsNot sb, "b IsNot sb")
#End If
#If Byte_ERRORS
            M(b IsNot s, "b IsNot s")
#End If
#If Byte_ERRORS
            M(b IsNot us, "b IsNot us")
#End If
#If Byte_ERRORS
            M(b IsNot i, "b IsNot i")
#End If
#If Byte_ERRORS
            M(b IsNot ui, "b IsNot ui")
#End If
#If Byte_ERRORS
            M(b IsNot l, "b IsNot l")
#End If
#If Byte_ERRORS
            M(b IsNot ul, "b IsNot ul")
#End If
#If Byte_ERRORS
            M(b IsNot dec, "b IsNot dec")
#End If
#If Byte_ERRORS
            M(b IsNot sng, "b IsNot sng")
#End If
#If Byte_ERRORS
            M(b IsNot dbl, "b IsNot dbl")
#End If
#If Byte_ERRORS
            M(b IsNot chr, "b IsNot chr")
#End If
#If Byte_ERRORS
            M(b IsNot str, "b IsNot str")
#End If
#If Byte_ERRORS
            M(b IsNot dt, "b IsNot dt")
#End If
#If Byte_ERRORS
            M(b IsNot dbnull, "b IsNot dbnull")
#End If
#If Byte_ERRORS
            M(b IsNot obj, "b IsNot obj")
#End If
#If SByte_ERRORS
            M(sb IsNot bool, "sb IsNot bool")
#End If
#If SByte_ERRORS
            M(sb IsNot b, "sb IsNot b")
#End If
#If SByte_ERRORS
            M(sb IsNot sb, "sb IsNot sb")
#End If
#If SByte_ERRORS
            M(sb IsNot s, "sb IsNot s")
#End If
#If SByte_ERRORS
            M(sb IsNot us, "sb IsNot us")
#End If
#If SByte_ERRORS
            M(sb IsNot i, "sb IsNot i")
#End If
#If SByte_ERRORS
            M(sb IsNot ui, "sb IsNot ui")
#End If
#If SByte_ERRORS
            M(sb IsNot l, "sb IsNot l")
#End If
#If SByte_ERRORS
            M(sb IsNot ul, "sb IsNot ul")
#End If
#If SByte_ERRORS
            M(sb IsNot dec, "sb IsNot dec")
#End If
#If SByte_ERRORS
            M(sb IsNot sng, "sb IsNot sng")
#End If
#If SByte_ERRORS
            M(sb IsNot dbl, "sb IsNot dbl")
#End If
#If SByte_ERRORS
            M(sb IsNot chr, "sb IsNot chr")
#End If
#If SByte_ERRORS
            M(sb IsNot str, "sb IsNot str")
#End If
#If SByte_ERRORS
            M(sb IsNot dt, "sb IsNot dt")
#End If
#If SByte_ERRORS
            M(sb IsNot dbnull, "sb IsNot dbnull")
#End If
#If SByte_ERRORS
            M(sb IsNot obj, "sb IsNot obj")
#End If
#If Int16_ERRORS
            M(s IsNot bool, "s IsNot bool")
#End If
#If Int16_ERRORS
            M(s IsNot b, "s IsNot b")
#End If
#If Int16_ERRORS
            M(s IsNot sb, "s IsNot sb")
#End If
#If Int16_ERRORS
            M(s IsNot s, "s IsNot s")
#End If
#If Int16_ERRORS
            M(s IsNot us, "s IsNot us")
#End If
#If Int16_ERRORS
            M(s IsNot i, "s IsNot i")
#End If
#If Int16_ERRORS
            M(s IsNot ui, "s IsNot ui")
#End If
#If Int16_ERRORS
            M(s IsNot l, "s IsNot l")
#End If
#If Int16_ERRORS
            M(s IsNot ul, "s IsNot ul")
#End If
#If Int16_ERRORS
            M(s IsNot dec, "s IsNot dec")
#End If
#If Int16_ERRORS
            M(s IsNot sng, "s IsNot sng")
#End If
#If Int16_ERRORS
            M(s IsNot dbl, "s IsNot dbl")
#End If
#If Int16_ERRORS
            M(s IsNot chr, "s IsNot chr")
#End If
#If Int16_ERRORS
            M(s IsNot str, "s IsNot str")
#End If
#If Int16_ERRORS
            M(s IsNot dt, "s IsNot dt")
#End If
#If Int16_ERRORS
            M(s IsNot dbnull, "s IsNot dbnull")
#End If
#If Int16_ERRORS
            M(s IsNot obj, "s IsNot obj")
#End If
#If UInt16_ERRORS
            M(us IsNot bool, "us IsNot bool")
#End If
#If UInt16_ERRORS
            M(us IsNot b, "us IsNot b")
#End If
#If UInt16_ERRORS
            M(us IsNot sb, "us IsNot sb")
#End If
#If UInt16_ERRORS
            M(us IsNot s, "us IsNot s")
#End If
#If UInt16_ERRORS
            M(us IsNot us, "us IsNot us")
#End If
#If UInt16_ERRORS
            M(us IsNot i, "us IsNot i")
#End If
#If UInt16_ERRORS
            M(us IsNot ui, "us IsNot ui")
#End If
#If UInt16_ERRORS
            M(us IsNot l, "us IsNot l")
#End If
#If UInt16_ERRORS
            M(us IsNot ul, "us IsNot ul")
#End If
#If UInt16_ERRORS
            M(us IsNot dec, "us IsNot dec")
#End If
#If UInt16_ERRORS
            M(us IsNot sng, "us IsNot sng")
#End If
#If UInt16_ERRORS
            M(us IsNot dbl, "us IsNot dbl")
#End If
#If UInt16_ERRORS
            M(us IsNot chr, "us IsNot chr")
#End If
#If UInt16_ERRORS
            M(us IsNot str, "us IsNot str")
#End If
#If UInt16_ERRORS
            M(us IsNot dt, "us IsNot dt")
#End If
#If UInt16_ERRORS
            M(us IsNot dbnull, "us IsNot dbnull")
#End If
#If UInt16_ERRORS
            M(us IsNot obj, "us IsNot obj")
#End If
#If Int32_ERRORS
            M(i IsNot bool, "i IsNot bool")
#End If
#If Int32_ERRORS
            M(i IsNot b, "i IsNot b")
#End If
#If Int32_ERRORS
            M(i IsNot sb, "i IsNot sb")
#End If
#If Int32_ERRORS
            M(i IsNot s, "i IsNot s")
#End If
#If Int32_ERRORS
            M(i IsNot us, "i IsNot us")
#End If
#If Int32_ERRORS
            M(i IsNot i, "i IsNot i")
#End If
#If Int32_ERRORS
            M(i IsNot ui, "i IsNot ui")
#End If
#If Int32_ERRORS
            M(i IsNot l, "i IsNot l")
#End If
#If Int32_ERRORS
            M(i IsNot ul, "i IsNot ul")
#End If
#If Int32_ERRORS
            M(i IsNot dec, "i IsNot dec")
#End If
#If Int32_ERRORS
            M(i IsNot sng, "i IsNot sng")
#End If
#If Int32_ERRORS
            M(i IsNot dbl, "i IsNot dbl")
#End If
#If Int32_ERRORS
            M(i IsNot chr, "i IsNot chr")
#End If
#If Int32_ERRORS
            M(i IsNot str, "i IsNot str")
#End If
#If Int32_ERRORS
            M(i IsNot dt, "i IsNot dt")
#End If
#If Int32_ERRORS
            M(i IsNot dbnull, "i IsNot dbnull")
#End If
#If Int32_ERRORS
            M(i IsNot obj, "i IsNot obj")
#End If
#If UInt32_ERRORS
            M(ui IsNot bool, "ui IsNot bool")
#End If
#If UInt32_ERRORS
            M(ui IsNot b, "ui IsNot b")
#End If
#If UInt32_ERRORS
            M(ui IsNot sb, "ui IsNot sb")
#End If
#If UInt32_ERRORS
            M(ui IsNot s, "ui IsNot s")
#End If
#If UInt32_ERRORS
            M(ui IsNot us, "ui IsNot us")
#End If
#If UInt32_ERRORS
            M(ui IsNot i, "ui IsNot i")
#End If
#If UInt32_ERRORS
            M(ui IsNot ui, "ui IsNot ui")
#End If
#If UInt32_ERRORS
            M(ui IsNot l, "ui IsNot l")
#End If
#If UInt32_ERRORS
            M(ui IsNot ul, "ui IsNot ul")
#End If
#If UInt32_ERRORS
            M(ui IsNot dec, "ui IsNot dec")
#End If
#If UInt32_ERRORS
            M(ui IsNot sng, "ui IsNot sng")
#End If
#If UInt32_ERRORS
            M(ui IsNot dbl, "ui IsNot dbl")
#End If
#If UInt32_ERRORS
            M(ui IsNot chr, "ui IsNot chr")
#End If
#If UInt32_ERRORS
            M(ui IsNot str, "ui IsNot str")
#End If
#If UInt32_ERRORS
            M(ui IsNot dt, "ui IsNot dt")
#End If
#If UInt32_ERRORS
            M(ui IsNot dbnull, "ui IsNot dbnull")
#End If
#If UInt32_ERRORS
            M(ui IsNot obj, "ui IsNot obj")
#End If
#If Int64_ERRORS
            M(l IsNot bool, "l IsNot bool")
#End If
#If Int64_ERRORS
            M(l IsNot b, "l IsNot b")
#End If
#If Int64_ERRORS
            M(l IsNot sb, "l IsNot sb")
#End If
#If Int64_ERRORS
            M(l IsNot s, "l IsNot s")
#End If
#If Int64_ERRORS
            M(l IsNot us, "l IsNot us")
#End If
#If Int64_ERRORS
            M(l IsNot i, "l IsNot i")
#End If
#If Int64_ERRORS
            M(l IsNot ui, "l IsNot ui")
#End If
#If Int64_ERRORS
            M(l IsNot l, "l IsNot l")
#End If
#If Int64_ERRORS
            M(l IsNot ul, "l IsNot ul")
#End If
#If Int64_ERRORS
            M(l IsNot dec, "l IsNot dec")
#End If
#If Int64_ERRORS
            M(l IsNot sng, "l IsNot sng")
#End If
#If Int64_ERRORS
            M(l IsNot dbl, "l IsNot dbl")
#End If
#If Int64_ERRORS
            M(l IsNot chr, "l IsNot chr")
#End If
#If Int64_ERRORS
            M(l IsNot str, "l IsNot str")
#End If
#If Int64_ERRORS
            M(l IsNot dt, "l IsNot dt")
#End If
#If Int64_ERRORS
            M(l IsNot dbnull, "l IsNot dbnull")
#End If
#If Int64_ERRORS
            M(l IsNot obj, "l IsNot obj")
#End If
#If UInt64_ERRORS
            M(ul IsNot bool, "ul IsNot bool")
#End If
#If UInt64_ERRORS
            M(ul IsNot b, "ul IsNot b")
#End If
#If UInt64_ERRORS
            M(ul IsNot sb, "ul IsNot sb")
#End If
#If UInt64_ERRORS
            M(ul IsNot s, "ul IsNot s")
#End If
#If UInt64_ERRORS
            M(ul IsNot us, "ul IsNot us")
#End If
#If UInt64_ERRORS
            M(ul IsNot i, "ul IsNot i")
#End If
#If UInt64_ERRORS
            M(ul IsNot ui, "ul IsNot ui")
#End If
#If UInt64_ERRORS
            M(ul IsNot l, "ul IsNot l")
#End If
#If UInt64_ERRORS
            M(ul IsNot ul, "ul IsNot ul")
#End If
#If UInt64_ERRORS
            M(ul IsNot dec, "ul IsNot dec")
#End If
#If UInt64_ERRORS
            M(ul IsNot sng, "ul IsNot sng")
#End If
#If UInt64_ERRORS
            M(ul IsNot dbl, "ul IsNot dbl")
#End If
#If UInt64_ERRORS
            M(ul IsNot chr, "ul IsNot chr")
#End If
#If UInt64_ERRORS
            M(ul IsNot str, "ul IsNot str")
#End If
#If UInt64_ERRORS
            M(ul IsNot dt, "ul IsNot dt")
#End If
#If UInt64_ERRORS
            M(ul IsNot dbnull, "ul IsNot dbnull")
#End If
#If UInt64_ERRORS
            M(ul IsNot obj, "ul IsNot obj")
#End If
#If Decimal_ERRORS
            M(dec IsNot bool, "dec IsNot bool")
#End If
#If Decimal_ERRORS
            M(dec IsNot b, "dec IsNot b")
#End If
#If Decimal_ERRORS
            M(dec IsNot sb, "dec IsNot sb")
#End If
#If Decimal_ERRORS
            M(dec IsNot s, "dec IsNot s")
#End If
#If Decimal_ERRORS
            M(dec IsNot us, "dec IsNot us")
#End If
#If Decimal_ERRORS
            M(dec IsNot i, "dec IsNot i")
#End If
#If Decimal_ERRORS
            M(dec IsNot ui, "dec IsNot ui")
#End If
#If Decimal_ERRORS
            M(dec IsNot l, "dec IsNot l")
#End If
#If Decimal_ERRORS
            M(dec IsNot ul, "dec IsNot ul")
#End If
#If Decimal_ERRORS
            M(dec IsNot dec, "dec IsNot dec")
#End If
#If Decimal_ERRORS
            M(dec IsNot sng, "dec IsNot sng")
#End If
#If Decimal_ERRORS
            M(dec IsNot dbl, "dec IsNot dbl")
#End If
#If Decimal_ERRORS
            M(dec IsNot chr, "dec IsNot chr")
#End If
#If Decimal_ERRORS
            M(dec IsNot str, "dec IsNot str")
#End If
#If Decimal_ERRORS
            M(dec IsNot dt, "dec IsNot dt")
#End If
#If Decimal_ERRORS
            M(dec IsNot dbnull, "dec IsNot dbnull")
#End If
#If Decimal_ERRORS
            M(dec IsNot obj, "dec IsNot obj")
#End If
#If Single_ERRORS
            M(sng IsNot bool, "sng IsNot bool")
#End If
#If Single_ERRORS
            M(sng IsNot b, "sng IsNot b")
#End If
#If Single_ERRORS
            M(sng IsNot sb, "sng IsNot sb")
#End If
#If Single_ERRORS
            M(sng IsNot s, "sng IsNot s")
#End If
#If Single_ERRORS
            M(sng IsNot us, "sng IsNot us")
#End If
#If Single_ERRORS
            M(sng IsNot i, "sng IsNot i")
#End If
#If Single_ERRORS
            M(sng IsNot ui, "sng IsNot ui")
#End If
#If Single_ERRORS
            M(sng IsNot l, "sng IsNot l")
#End If
#If Single_ERRORS
            M(sng IsNot ul, "sng IsNot ul")
#End If
#If Single_ERRORS
            M(sng IsNot dec, "sng IsNot dec")
#End If
#If Single_ERRORS
            M(sng IsNot sng, "sng IsNot sng")
#End If
#If Single_ERRORS
            M(sng IsNot dbl, "sng IsNot dbl")
#End If
#If Single_ERRORS
            M(sng IsNot chr, "sng IsNot chr")
#End If
#If Single_ERRORS
            M(sng IsNot str, "sng IsNot str")
#End If
#If Single_ERRORS
            M(sng IsNot dt, "sng IsNot dt")
#End If
#If Single_ERRORS
            M(sng IsNot dbnull, "sng IsNot dbnull")
#End If
#If Single_ERRORS
            M(sng IsNot obj, "sng IsNot obj")
#End If
#If Double_ERRORS
            M(dbl IsNot bool, "dbl IsNot bool")
#End If
#If Double_ERRORS
            M(dbl IsNot b, "dbl IsNot b")
#End If
#If Double_ERRORS
            M(dbl IsNot sb, "dbl IsNot sb")
#End If
#If Double_ERRORS
            M(dbl IsNot s, "dbl IsNot s")
#End If
#If Double_ERRORS
            M(dbl IsNot us, "dbl IsNot us")
#End If
#If Double_ERRORS
            M(dbl IsNot i, "dbl IsNot i")
#End If
#If Double_ERRORS
            M(dbl IsNot ui, "dbl IsNot ui")
#End If
#If Double_ERRORS
            M(dbl IsNot l, "dbl IsNot l")
#End If
#If Double_ERRORS
            M(dbl IsNot ul, "dbl IsNot ul")
#End If
#If Double_ERRORS
            M(dbl IsNot dec, "dbl IsNot dec")
#End If
#If Double_ERRORS
            M(dbl IsNot sng, "dbl IsNot sng")
#End If
#If Double_ERRORS
            M(dbl IsNot dbl, "dbl IsNot dbl")
#End If
#If Double_ERRORS
            M(dbl IsNot chr, "dbl IsNot chr")
#End If
#If Double_ERRORS
            M(dbl IsNot str, "dbl IsNot str")
#End If
#If Double_ERRORS
            M(dbl IsNot dt, "dbl IsNot dt")
#End If
#If Double_ERRORS
            M(dbl IsNot dbnull, "dbl IsNot dbnull")
#End If
#If Double_ERRORS
            M(dbl IsNot obj, "dbl IsNot obj")
#End If
#If Char_ERRORS
            M(chr IsNot bool, "chr IsNot bool")
#End If
#If Char_ERRORS
            M(chr IsNot b, "chr IsNot b")
#End If
#If Char_ERRORS
            M(chr IsNot sb, "chr IsNot sb")
#End If
#If Char_ERRORS
            M(chr IsNot s, "chr IsNot s")
#End If
#If Char_ERRORS
            M(chr IsNot us, "chr IsNot us")
#End If
#If Char_ERRORS
            M(chr IsNot i, "chr IsNot i")
#End If
#If Char_ERRORS
            M(chr IsNot ui, "chr IsNot ui")
#End If
#If Char_ERRORS
            M(chr IsNot l, "chr IsNot l")
#End If
#If Char_ERRORS
            M(chr IsNot ul, "chr IsNot ul")
#End If
#If Char_ERRORS
            M(chr IsNot dec, "chr IsNot dec")
#End If
#If Char_ERRORS
            M(chr IsNot sng, "chr IsNot sng")
#End If
#If Char_ERRORS
            M(chr IsNot dbl, "chr IsNot dbl")
#End If
#If Char_ERRORS
            M(chr IsNot chr, "chr IsNot chr")
#End If
#If Char_ERRORS
            M(chr IsNot str, "chr IsNot str")
#End If
#If Char_ERRORS
            M(chr IsNot dt, "chr IsNot dt")
#End If
#If Char_ERRORS
            M(chr IsNot dbnull, "chr IsNot dbnull")
#End If
#If Char_ERRORS
            M(chr IsNot obj, "chr IsNot obj")
#End If
#If String_ERRORS
            M(str IsNot bool, "str IsNot bool")
#End If
#If String_ERRORS
            M(str IsNot b, "str IsNot b")
#End If
#If String_ERRORS
            M(str IsNot sb, "str IsNot sb")
#End If
#If String_ERRORS
            M(str IsNot s, "str IsNot s")
#End If
#If String_ERRORS
            M(str IsNot us, "str IsNot us")
#End If
#If String_ERRORS
            M(str IsNot i, "str IsNot i")
#End If
#If String_ERRORS
            M(str IsNot ui, "str IsNot ui")
#End If
#If String_ERRORS
            M(str IsNot l, "str IsNot l")
#End If
#If String_ERRORS
            M(str IsNot ul, "str IsNot ul")
#End If
#If String_ERRORS
            M(str IsNot dec, "str IsNot dec")
#End If
#If String_ERRORS
            M(str IsNot sng, "str IsNot sng")
#End If
#If String_ERRORS
            M(str IsNot dbl, "str IsNot dbl")
#End If
#If String_ERRORS
            M(str IsNot chr, "str IsNot chr")
#End If
            expected_tc = TypeCode.Boolean
            M(str IsNot str, "str IsNot str")
#If String_ERRORS
            M(str IsNot dt, "str IsNot dt")
#End If
#If String_ERRORS
            M(str IsNot dbnull, "str IsNot dbnull")
#End If
            expected_tc = TypeCode.Boolean
            M(str IsNot obj, "str IsNot obj")
#If DateTime_ERRORS
            M(dt IsNot bool, "dt IsNot bool")
#End If
#If DateTime_ERRORS
            M(dt IsNot b, "dt IsNot b")
#End If
#If DateTime_ERRORS
            M(dt IsNot sb, "dt IsNot sb")
#End If
#If DateTime_ERRORS
            M(dt IsNot s, "dt IsNot s")
#End If
#If DateTime_ERRORS
            M(dt IsNot us, "dt IsNot us")
#End If
#If DateTime_ERRORS
            M(dt IsNot i, "dt IsNot i")
#End If
#If DateTime_ERRORS
            M(dt IsNot ui, "dt IsNot ui")
#End If
#If DateTime_ERRORS
            M(dt IsNot l, "dt IsNot l")
#End If
#If DateTime_ERRORS
            M(dt IsNot ul, "dt IsNot ul")
#End If
#If DateTime_ERRORS
            M(dt IsNot dec, "dt IsNot dec")
#End If
#If DateTime_ERRORS
            M(dt IsNot sng, "dt IsNot sng")
#End If
#If DateTime_ERRORS
            M(dt IsNot dbl, "dt IsNot dbl")
#End If
#If DateTime_ERRORS
            M(dt IsNot chr, "dt IsNot chr")
#End If
#If DateTime_ERRORS
            M(dt IsNot str, "dt IsNot str")
#End If
#If DateTime_ERRORS
            M(dt IsNot dt, "dt IsNot dt")
#End If
#If DateTime_ERRORS
            M(dt IsNot dbnull, "dt IsNot dbnull")
#End If
#If DateTime_ERRORS
            M(dt IsNot obj, "dt IsNot obj")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot bool, "dbnull IsNot bool")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot b, "dbnull IsNot b")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot sb, "dbnull IsNot sb")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot s, "dbnull IsNot s")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot us, "dbnull IsNot us")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot i, "dbnull IsNot i")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot ui, "dbnull IsNot ui")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot l, "dbnull IsNot l")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot ul, "dbnull IsNot ul")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot dec, "dbnull IsNot dec")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot sng, "dbnull IsNot sng")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot dbl, "dbnull IsNot dbl")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot chr, "dbnull IsNot chr")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot str, "dbnull IsNot str")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot dt, "dbnull IsNot dt")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot dbnull, "dbnull IsNot dbnull")
#End If
#If DBNull_ERRORS
            M(dbnull IsNot obj, "dbnull IsNot obj")
#End If
#If Object_ERRORS
            M(obj IsNot bool, "obj IsNot bool")
#End If
#If Object_ERRORS
            M(obj IsNot b, "obj IsNot b")
#End If
#If Object_ERRORS
            M(obj IsNot sb, "obj IsNot sb")
#End If
#If Object_ERRORS
            M(obj IsNot s, "obj IsNot s")
#End If
#If Object_ERRORS
            M(obj IsNot us, "obj IsNot us")
#End If
#If Object_ERRORS
            M(obj IsNot i, "obj IsNot i")
#End If
#If Object_ERRORS
            M(obj IsNot ui, "obj IsNot ui")
#End If
#If Object_ERRORS
            M(obj IsNot l, "obj IsNot l")
#End If
#If Object_ERRORS
            M(obj IsNot ul, "obj IsNot ul")
#End If
#If Object_ERRORS
            M(obj IsNot dec, "obj IsNot dec")
#End If
#If Object_ERRORS
            M(obj IsNot sng, "obj IsNot sng")
#End If
#If Object_ERRORS
            M(obj IsNot dbl, "obj IsNot dbl")
#End If
#If Object_ERRORS
            M(obj IsNot chr, "obj IsNot chr")
#End If
            expected_tc = TypeCode.Boolean
            M(obj IsNot str, "obj IsNot str")
#If Object_ERRORS
            M(obj IsNot dt, "obj IsNot dt")
#End If
#If Object_ERRORS
            M(obj IsNot dbnull, "obj IsNot dbnull")
#End If
            expected_tc = TypeCode.Boolean
            M(obj IsNot obj, "obj IsNot obj")
        If failures > 0 Then Return 1
    Catch ex As Exception
        Console.WriteLine ("Exception: {0}", ex)
        Return 2
    End Try
    Return 0
    End Function
End Class
