Imports System
Imports System.Collections
Imports System.Reflection

Namespace GenericFieldAccess1
    Class G(Of T) 
    End Class

    Class Test
        Shared Function Main() As Integer

        End Function
    End Class
End Namespace