Class StructureMembers8
	Structure S
		Dim I As Integer

		Sub Method ()
			Dim Cl As New StructureMembers8 (Me)
		End Sub
	End Structure

	Sub New (S As S)
    End Sub
    Shared Sub Main()

    End Sub
End Class
