Class FailsWithDelegateParameter
    Shared Sub Main
    End Sub

    Public Function BeginInvoke(method As Delegate) As IAsyncResult
        throw New NotImplementedException
    End Function
End Class
