Class Partial9
   Shared Function Main() As Integer
       Dim result As Boolean

       If result = False Then
           System.Console.WriteLine("FAIL Partial9")
           System.Console.WriteLine("(detailed message)")
           Return 1
       End If
   End Function
End Class
