Imports System
Imports System.Collections
Imports System.Reflection

Namespace FieldAccess1
    Class T(Of X)
        Public V As Integer = 1
    End Class
    Public Class Test
        Shared Sub Main()

        End Sub
    End Class
End Namespace