Imports System
Imports System.Collections
Imports System.Reflection

Namespace ByRef10
    Class Test
        Shared Function Tester(ByRef V() As Integer) As Integer
            Return v(0)
        End Function

        Shared Function Main() As Integer

        End Function
    End Class
End Namespace