Imports System.Collections

Public Class Generic8
    Shared Function Main() As Integer
        Dim allModuleTypes As New Generic.List(Of String)
        Dim found As New generic.list(Of Object)
        If False Then
            found.add(allmoduletypes.item(0))
            Return 1
        Else
            Return 0
        End If
    End Function

End Class
