Imports System
Imports System.Collections
Imports System.Reflection

Namespace For1
    Class Test
        Shared Function Main() As Integer
            For d As Double = 0 To 3

            Next
        End Function
    End Class
End Namespace