' 
' Visual Basic.Net Compiler
' Copyright (C) 2004 - 2010 Rolf Bjarne Kvinge, RKvinge@novell.com
' 
' This library is free software; you can redistribute it and/or
' modify it under the terms of the GNU Lesser General Public
' License as published by the Free Software Foundation; either
' version 2.1 of the License, or (at your option) any later version.
' 
' This library is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
' Lesser General Public License for more details.
' 
' You should have received a copy of the GNU Lesser General Public
' License along with this library; if not, write to the Free Software
' Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
' 
''' <summary>
''' Enum of built in data types. 
''' </summary>
''' <remarks></remarks>
Public Enum BuiltInDataTypes
    [Boolean] = KS.Boolean
    [Byte] = KS.Byte
    [Char] = KS.Char
    [Date] = KS.Date
    [Decimal] = KS.Decimal
    [Double] = KS.Double
    [Integer] = KS.Integer
    [Long] = KS.Long
    [Object] = KS.Object
    [Short] = KS.Short
    [Single] = KS.Single
    [String] = KS.String
    [SByte] = KS.[SByte]
    [UShort] = KS.[UShort]
    [UInteger] = KS.[UInteger]
    [ULong] = KS.[ULong]
End Enum