Imports System.Windows.Forms

Public Class Form1
    Dim i As Integer
    Shared Sub Main()

    End Sub
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles i.Load

    End Sub
End Class