'MemberAccessExpression: E.I
'If "E" is classified as a namespace and "I" is the name of an accessible member of 
'that namespace, then the result is that member. The result is classified as
'a namespace of a type depending on the member.
Class MemberAccessExpression3
    Sub Test()
        Dim o As System.Object
    End Sub
End Class