Imports System

Module Identifier
    Function _abc() As Object
    End Function

    Function xys2() As Object
    End Function

    '' escaped identifier
    Sub [sub]()
    End Sub

    Dim [function] As Int16

    Function Main() As Integer
    End Function
End Module
