Class AndConstantExpression1
	Const const1 As Boolean = True And False
	Const const2 As Boolean = True And True
	Const const3 As Boolean = False And True
	Const const4 As Boolean = False And False
End Class
