
Module VoyagerBrowser

    Sub Main()
        Dim FowardButton As Object
      
        FowardButton = VoyagerBrowser.GoFoward
      
    End Sub

    Public ReadOnly Property GoForward() As String
        Get
            Return Nothing
        End Get
    End Property


End Module
