'test conditional compilation expressions
'RuleIdentifier
#const testIdentInt = 1
#const testIdentString = "abc"
#const testIdentDouble = 3.14
#const testIdentNothing = nothing
#const testIdentDate = #1/1/2002#


'RuleExponent
#const testExp = 2 ^ 2
'RuleUnaryNegative
#const testUnaryNeg = -2
'RuleMultiplication
#const testMult = 2*2
'RuleRealDiv
#const testRealDiv = 2 / 3
'RuleIntDiv
#const testIntDiv = 2 \ 3
'RuleMod
#const testMod = 2 mod 3
'RuleAdd
#const testAdd = 3 + 4
'RuleSubtract 
#const testSub = 3 - 4
'RuleConcat+
#const testConcat1 = "a" + "b"

'RuleConcat&
#const testConcat2 = "c" & "d"
'RuleShiftLeft
#const testShl = 20 << 2
'RuleShiftRight
#const testShr = 20 >> 2

'
'
'
'