Public Class TypeNameResolution3
    Public Class MimeType
        Public Const [Binary] As String = "Binary"
    End Class

    Shared Sub S(Optional ByVal MimeType As String = MimeType.Binary)

    End Sub

End Class