'Test structure access
Class StructureAccess1
    Sub Main()
        '        Dim bo As Boolean
        '        Dim by As Byte
        '        Dim ch As Char
        '        Dim dt As Date
        '        Dim de As Decimal
        '        Dim db As Double
        '        Dim i32 As Integer
        '        Dim i16 As Short
        '        Dim i64 As Long
        '        Dim str As String
        '        Dim si As Single
        '        Dim ui16 As UShort
        '        Dim ui32 As UInteger
        '        Dim ui64 As ULong

        'dim dt2 as date
        '        'dt = date.now
        '        dt2 = Date.Now
    End Sub

End Class