Class NewExpression4
	Sub Test()
		test(new newexpression4(1,,2))
	End Sub
	sub Test(var as newexpression4)
	end sub

	sub new(optional var1 as integer = 1, optional var2 as integer = 2, optional var3 as integer = 3)
		
	end sub
End Class