'Author:
'   V. Sudharsan (vsudharsan@novell.com)
'
' (C) 2005 Novell, Inc.

MustInherit Class A
    MustInherit Class B
    End Class
End Class

Class C
    Inherits A
End Class

Module InheritanceN
    Function Main() As Integer
    End Function
End Module
