Option Strict Off
Imports System
Module RegionDirectives
#Region "Main"
    Function Main() As Integer


    End Function

#Region "S"
    Function S()



    End Function
#End Region

#End Region
End Module

