Imports System
Imports System.Collections
Imports System.Reflection

Namespace MethodInvocation11
    Class Test
        Shared Function T() As Boolean
            Return Nothing
        End Function

        Shared Function Main() As Integer
            Return 0
        End Function
    End Class
End Namespace